
%macro idb_prepare;
LIBNAME IDB ACCESS
            "&idb_file.";
%let cond=work.cond;
%let ppsa=work.ppsa;
%let eus=work.eus;
%let attribute_tablex=idb.&attribute_namex.;
%let attribute_tabley=idb.&attribute_namey.;

data cond_temp(where=(cond_wt>0));
  set idb.cond(keep = stratum_id stratum_vol_id cond_wt cond_id plot_id glc_group
                      state cnty plot data_source);

  if data_source = "R5" then do;
    plot_id = plot_id * 10000+ stratum_id;
/*	cond_wt = subplot_count_cond/5;*/
  end;

run;

data stratum_temp;
  set idb.stratum(keep = stratum_id scheme unit field_plot_count photo_point_count);
  if unit = . then unit = 1;
run;


proc sort data = cond_temp;
  by stratum_id;
run;
proc sort data = stratum_temp;
  by stratum_id;
run;

data ppsa_temp_1(where=(cond_wt>0 and scheme ne .));
  merge stratum_temp cond_temp;
  by stratum_id;
run;

data cond_temp;
  set cond_temp;
  if stratum_id = stratum_vol_id then delete;

  if data_source = "FIACA" then do;
    if stratum_vol_id > 0 and stratum_id ne stratum_vol_id then do;
      stratum_id = stratum_vol_id;
	  plot_id = plot_id;
	end;
  end;
  else do;
    stratum_id = stratum_vol_id;
  end;
  if stratum_id = . then delete;
  if data_source = "R5" then plot_id = plot_id * 10000+ stratum_id;
run;

proc sort data = cond_temp;
  by stratum_id;
run;

data ppsa_temp_2(where=(cond_wt>0 and scheme ne .));
  merge stratum_temp cond_temp(where=(cond_wt>0));
  by stratum_id;
run;

data ppsa_temp_3;
  set ppsa_temp_1 ppsa_temp_2;
  drop stratum_vol_id glc_group;
  statecd = state;
  plot = plot_id;
  evalid = scheme;
  estunit = unit;
  countycd = cnty;
  subcycle = 1;
  cycle = 1;
  stratumcd = stratum_id;
  cycle = 1;
  unitcd=1;
  rscd = 26;
run;

proc sort data = ppsa_temp_3;
  by scheme stratum_id plot_id;
run;

data ppsa;
  set ppsa_temp_3;
  by scheme stratum_id plot_id;
  stratumcd = stratum_id;
  if last.plot_id and not(plot_id = . and last.stratum_id) then output;
  drop cond_wt photo_point_count;
run;
proc sort data = ppsa;
  by scheme stratumcd;
run;


data ppsa;
  set ppsa;
  by scheme stratumcd;
  retain count_plot 0;

  if first.stratumcd then count_plot = 1;
  else count_plot = count_plot+1;
run;



proc sql noprint;
  select max(plot)
    into :max_plot
	  from ppsa
	  where  evalid <> 2
	  ;
quit;

data ppsa(drop=i run_plot cond_id)
     temp_cond(keep=data_source cond_id plot_id plot stratumcd statecd countycd state cnty subcycle cycle);
  set ppsa;
    retain run_plot -1; /*%eval(&max_plot. + 1);*/
    by evalid stratumcd plot;
    output ppsa;
	if evalid <> 2 then do;
	  if last.stratumcd then do;
	    if count_plot < field_plot_count then
	      do i=count_plot+1 to field_plot_count;
		    plot=run_plot;
  		    plot_id = run_plot;
		    cond_id = plot_id;
/*		    run_plot = run_plot+1;*/
		    run_plot = run_plot-1;
		    output ppsa;
		    output temp_cond;
		  end;
		end;
	end;
run;

%if "&dwm_cond_source." ne "" %then %do;
proc sql;
  create table down_wood_temp as
    select distinct cond_id, mean (tl_cond_hd_ft) as tl_cond_hd_ft
	  from idb.down_wood
	  group by cond_id;
quit;

data down_wood_temp (drop = _tl_cond_hd_ft_);
  set down_wood_temp;
  tl_cond_hd_ft = _tl_cond_hd_ft_;
run;
%end;


data cond;
  set idb.cond(where = (cond_wt >0));

  if data_source = "R5" then do;
    plot_id = plot_id * 10000+ stratum_id;
	cond_wt = subplot_count_cond/5;
  end;
  if data_source = "FIACA" and stratum_vol_id > 0 and stratum_id ne stratum_vol_id then do;
    stratum_id = stratum_vol_id;
  end;
  statecd = state;
  plot = plot_id;
  countycd = cnty;
  subcycle = 1;
  cycle = 1;
  stratumcd = stratum_id;
  unitcd=1;
  rscd = 26;

run;


data cond;
  set idb.cond(where = (cond_wt >0));

  if data_source = "R5" then do;
    plot_id = plot_id * 10000+ stratum_id;
	cond_wt = subplot_count_cond/5;
  end;
  if data_source = "FIACA" and stratum_vol_id > 0 and stratum_id ne stratum_vol_id then do;
    stratum_id = stratum_vol_id;
  end;
  statecd = state;
  plot = plot_id;
  countycd = cnty;
  subcycle = 1;
  cycle = 1;
  stratumcd = stratum_id;
  unitcd=1;
  rscd = 26;

run;


data temp_cond;
  set temp_cond;
  glc_group = "NF";
  cond_wt = 1;
  rscd = 26;
  cond_class=1;
  unitcd=1;
  cycle = 1;
run;

data cond;
  set cond temp_cond;
run;

proc sort data = cond;
  by cond_id;
run;

%if "&dwm_cond_source." ne "" %then %do;
data cond;
  merge cond down_wood_temp;
  by cond_id;
%end;

data cond;
  set cond;
  statecd = state;
run;



data stratum_temp2;
  set idb.stratum;
  if unit = . then unit = 1;
  if adjfactor_r5 = . then adjfactor_r5 = .;
run;

proc sql;
   create table eus as
     select distinct (26) as rscd, s.scheme as evalid, s.unit as estunit, 
            s.stratum_id as stratumcd, s.stratum_acres, sum.areatot_scheme,
			s.field_plot_count as p2pointcnt,
	        s.photo_point_count as p1pointcnt,
			sum.p1pointcnt_eu,
			s.AdjFactor_R5
	   from stratum_temp2 s,
	        (
			select distinct s.scheme, s.unit,
                            sum(s.stratum_acres) as areatot_scheme,
	                        sum(s.photo_point_count) as p1pointcnt_eu
	             from stratum_temp2 s
	                 group by s.scheme, s.unit
			) as sum
			where s.scheme = sum.scheme and
			      s.unit   = sum.unit /*and
			s.field_plot_count > 1*/
	     ;
quit;


data eus;
  set eus;
  rename _p1pointcnt_eu_  = p1pointcnt_eu
         _areatot_scheme_ = areatot_scheme
         /*_arealand_eu_=arealand_eu
		 _stratumcd_=stratumcd
         _evalid_=evalid
		 _estunit_=estunit
         _evaldescr_=evaldescr*/;
  if adjfactor_r5=. then adjfactor_r5 = 1;
run;
data eus;
  set eus;
  if p1pointcnt = . then p1pointcnt = 1;
  if p1pointcnt_eu = . then p1pointcnt_eu = 1;
/*This is a temporary fix until the stratum table is consistent in IDB and
  the plots are assigned to the right stratum.*/
*  p1pointcnt = 1;
*  p1pointcnt_eu = 1;
/*end of temporary fix*/

  if p1pointcnt_eu = 1 then do; 
     area_used   = stratum_acres;
	 arealand_eu = stratum_acres;
	 areatot_eu  = stratum_acres;
	 end;
  else do;
     area_used   = areatot_scheme;
	 arealand_eu = areatot_scheme;
	 areatot_eu  = areatot_scheme;
	 end;
run;

data eus;
  set eus;
  select (evalid);
    when (1)  evaldescr="FIA California 1991                                              ";
    when (2)  evaldescr="R5 National Forest                                               ";
    when (3)  evaldescr="FIA Eastern Oregon 1998 Area Stratification                      ";
    when (4)  evaldescr="FIA Eastern Washington 1991                                      ";
    when (5)  evaldescr="FIA Western Oregon 1995 Area Stratification                      ";
    when (6)  evaldescr="FIA Western Washington 1988                                      ";
    when (7)  evaldescr="Oregon Washington, National Forest Wilderness/Non-Wilderness     ";
    when (8)  evaldescr="BLM Western Oregon                                               ";
    when (9)  evaldescr="FIA Western Oregon 1995 Volume Stratification                    ";
    when (10) evaldescr="FIA Eastern Oregon 1998 Volume Stratificaiton                    ";
    when (11) evaldescr="California 1994 Woodland                                         ";
    when (12) evaldescr="Eastern Oregon 1999 Juniper                                      ";
    when (13) evaldescr="Kaniksu National Forest Washington                               ";
    when (14) evaldescr="Toiyabe National Forest California                               ";

    when (20) evaldescr="FIA Western Washington 2000 Area Stratification                  ";
    when (21) evaldescr="FIA Western Washington 2000 Volume Stratification                ";
    when (22) evaldescr="FIA Eastern Washington 2001 Area Stratification                  ";
    when (23) evaldescr="FIA Eastern Washington 2001 Volume Stratification                ";

    when (24) evaldescr="RMRS R1 Washington                                               ";
    when (25) evaldescr="R6 CVS                                                           ";
    otherwise evaldescr="                                                                 ";
  end;
*  if evalid = 2 then delete;
run;




%mend;

/*
%idb_prepare;
*/
