/*you only need to change the following directory ones. Save the SAS file and run it.*/
/* SAS program is written by Olaf Kuegler, USDA Forest Service, PNW research station, Portland Oregon
   email: okuegler@fs.fed.us
*/   




%let tabeling_dir=&work_dir.\tabling_program;



*options mlogic mprint symbolgen;
options nomlogic nomprint nosymbolgen;

%let version=1.0C;

libname proclib "&tabeling_dir.";
%include "&tabeling_dir.\import data.sas" /LRECL= 5000;
%include "&tabeling_dir.\in.sas" /LRECL= 5000;
%include "&tabeling_dir.\all_programs.sas" /LRECL= 5000;
%include "&tabeling_dir.\predefined_groups.sas" /LRECL= 5000;
%include "&tabeling_dir.\setup species group.sas" /LRECL= 5000;
%include "&tabeling_dir.\setup forest type.sas" /LRECL= 5000;
%include "&tabeling_dir.\prepare_idb.sas" /LRECL= 5000;
%include "&tabeling_dir.\calculate access denied and number of plots per stratum.sas" /LRECL= 5000;
%include "&tabeling_dir.\export_column_format.sas" /LRECL= 5000;


%macro define_global_macros;

%if not(%symexist(delimiter)) %then %do;
   %global delimiter;
   %let delimiter='|';
%end;
%if not(%symexist(exclude_col)) %then %do;
  %global exclude_col;
  %let exclude_col=0;
%end;
%if not(%symexist(exclude_row)) %then %do;
  %global exclude_row;
  %let exclude_row=0;
%end;
%if not(%symexist(backgroundcolor)) %then %do;
  %global backgroundcolor;
  %let backgroundcolor=white;
%end;

%if not(%symexist(nims_load_sel)) %then %do;
  %global nims_load_sel;
  %let nims_load_sel=;
%end;
%if not(%symexist(nims_production_sel)) %then %do;
  %global nims_production_sel;
  %let nims_production_sel=;
%end;
%if not(%symexist(access_sel)) %then %do;
  %global access_sel;
  %let access_sel=;
%end;
%if not(%symexist(sas_sel)) %then %do;
  %global sas_sel;
  %let sas_sel=;
%end;
%if not(%symexist(access_dir)) %then %do;
  %global access_dir;
  %let access_dir=&tabeling_dir.;
%end;

%if not(%symexist(ppsa_sort_by_var)) %then %do;
  %global ppsa_sort_by_var;
%end;

%if not(%symexist(use_adj_db)) %then %do;
  %global use_adj_db;
%end;

%if not(%symexist(datasource_nims_40)) %then %do;
  %global datasource_nims_40;
%end;

%if not(%symexist(datasource_fiadb_40)) %then %do;
  %global datasource_fiadb_40;
%end;

%if not(%symexist(idb_dir)) %then %do;
  %global idb_dir;
  %let idb_dir=&tabeling_dir.;
%end;
%if not(%symexist(idb_file)) %then %do;
  %global idb_file;
  %let idb_file=;
%end;

%if not(%symexist(cond_name)) %then %do;
  %global cond_name;
  %let cond_name=cond;
%end;
%if not(%symexist(cwd_name)) %then %do;
  %global cwd_name cwd cwd_source;
  %let cwd_name= P2_DWM_CWD_VW;
%end;
%if not(%symexist(fwd_name)) %then %do;
  %global fwd_name fwd fwd_source;
  %let fwd_name= P2_DWM_FWD_VW;
%end;
%if not(%symexist(dwm_cond_name)) %then %do;
  %global dwm_cond_name dwm dwm_cond_source;
  %let dwm_cond_name= P2_DWM_COND_CALC_VW;
%end;
%if not(%symexist(attribute_type)) %then %do;
  %global attribute_type;
  %let attribute_type= TREE;
%end;
%if not(%symexist(adj_factor)) %then %do;
  %global adj_factor;
  %let adj_factor= eus.pmh_cond;
%end;
%if not(%symexist(ppsa_name)) %then %do;
   %global ppsa_name;
   %let ppsa_name=plot_pop_stratum_assgn;
%end;
%if not(%symexist(eus_name)) %then %do;
   %global eus_name;
   %let eus_name=estn_unit_stratum;
%end;


%if not(%symexist(pop_plot_stratum_assgn_name)) %then %do;
   %global pop_plot_stratum_assgn_name;
   %let pop_plot_stratum_assgn_name=pop_plot_stratum_assgn;
%end;
%if not(%symexist(pop_stratum_name)) %then %do;
   %global pop_stratum_name;
   %let pop_stratum_name=pop_stratum;
%end;
%if not(%symexist(pop_eval_name)) %then %do;
   %global pop_eval_name;
   %let pop_eval_name=pop_eval;
%end;
%if not(%symexist(pop_estn_unit_name)) %then %do;
   %global pop_estn_unit_name;
   %let pop_estn_unit_name=pop_estn_unit;
%end;

   


%if not(%symexist(attribute_namex)) %then %do;
   %global attribute_namex;
   %let attribute_namex=tree;
%end;
%if not(%symexist(attribute_namey)) %then %do;
   %global attribute_namey;
   %let attribute_namey=tree;
%end;
%if not(%symexist(ftype_name)) %then %do;
   %global ftype_name;
   %let ftype_name=ftype_name;
%end;
%if not(%symexist(species_group_name)) %then %do;
   %global species_group_name;
   %let species_group_name=REF_SPECIES_group;
%end;
%if not(%symexist(variable_directionx)) %then %do;
   %global variable_directionx;
   %let variable_directionx=1;
%end;
%if not(%symexist(variable_directiony)) %then %do;
   %global variable_directiony;
   %let variable_directiony=1;
%end;
%if not(%symexist(page_orientation)) %then %do;
   %global page_orientation;
   %let page_orientation=1;
%end;
%if not(%symexist(print_date)) %then %do;
   %global print_date;
   %let print_date=1;
%end;
%if not(%symexist(use_idb)) %then %do;
   %global use_idb;
   %let use_idb=;
%end;

%if not(%symexist(use_cond_adj)) %then %do;
   %global use_cond_adj;
   %let use_cond_adj=;
%end;

%if not(%symexist(num_IDB_files)) %then %do;
   %global num_IDB_files;
   %let num_IDB_files=;
%end;

%if not(%symexist(num_SAS_files)) %then %do;
   %global num_SAS_files;
   %let num_SAS_files=;
%end;

%if not(%symexist(num_ACCESS_files)) %then %do;
   %global num_ACCESS_files;
   %let num_ACCESS_files=;
%end;

%if not(%symexist(census_water)) %then %do;
   %global census_water;
   %let census_water=1;
%end;

%if not(%symexist(sum_across_evalid)) %then %do;
   %global sum_across_evalid;
   %let sum_across_evalid=1;
%end;
%if not(%symexist(subtract)) %then %do;
   %global subtract;
   %let subtract=.;
%end;
%if not(%symexist(ratio_y_n)) %then %do;
   %global ratio_y_n;
   %let ratio_y_n=2;
%end;
%if not(%symexist(r_var)) %then %do;
   %global r_var;
   %let r_var=x y;
%end;

%if not(%symexist(fit_one_page)) %then %do;
   %global fit_one_page;
   %let fit_one_page=2;
%end;
%if not(%symexist(close_excel_sheet)) %then %do;
   %global close_excel_sheet;
   %let close_excel_sheet=2;
%end;
%if not(%symexist(orange_value)) %then %do;
   %global orange_value;
   %let orange_value=3;
%end;
%if not(%symexist(red_value)) %then %do;
   %global red_value;
   %let red_value=4;
%end;
%if not(%symexist(color_large_value)) %then %do;
   %global color_large_value;
   %let color_large_value=2;
%end;

%if not(%symexist(export_column_format_yes)) %then %do;
   %global export_column_format_yes;
   %let export_column_format_yes=2;
%end;
%if not(%symexist(evaldescr_yes)) %then %do;
   %global evaldescr_yes;
   %let evaldescr_yes=2;
%end;
%if not(%symexist(paper_size)) %then %do;
   %global paper_size;
   %let paper_size=1;
%end;
%if not(%symexist(number_of_digits)) %then %do;
   %global number_of_digits;
   %let number_of_digits=0;
%end;
%if not(%symexist(excel_font)) %then %do;
   %global excel_font;
   %let excel_font=MS Sans Serif;
%end;
%if not(%symexist(excel_font_size)) %then %do;
   %global excel_font_size;
   %let excel_font_size=10;
%end;
%if not(%symexist(excel_title_font_size)) %then %do;
   %global excel_title_font_size;
   %let excel_title_font_size=10;
%end;
%if not(%symexist(rowname_width)) %then %do;
   %global rowname_width rowname2_width;
%end;


%if not(%symexist(overwrite_file)) %then %do;
   %global overwrite_file;
   %let overwrite_file=2;
%end;

%if not(%symexist(rounding_threshold)) %then %do;
   %global rounding_threshold;
   %let rounding_threshold=0;
%end;

%if not(%symexist(num_col_break)) %then %do;
   %global num_col_break;
   %let num_col_break=0 ;
%end;

%global add_tablex add_connectionx add_tabley add_connectiony; 

%global Title title2 max_row_num max_col_num max_variable_of_interest
          restrictionsx restrictionsy units exclude_col exclude_row backgroundcolor 
          one_datasource sas_library access_file access_file_name update_all_files import_now
          display_tables rtf_location rtf_dir
          evalid evaldescr rscd num_avail_evalid avail_evalid avail_evaldesc
          row_var_name_out col_var_name_out glenn variable_directionx variable_directiony
          pre_defined_row_col predefined_row predefined_col predefined_restriction
          nims_schema let nims_user let nims_password;
  %global cond_source cwd_source ppsa_source eus_source attribute_sourcex attribute_sourcey ftype_source species_group_source;
  %global pop_plot_stratum_assgn_source pop_stratum_source pop_eval_source pop_estn_unit_source;
  %global pop_plot_stratum_assgn pop_stratum pop_eval pop_estn_unit;
  %global cond ppsa eus attribute_tablex attribute_tabley ftype species_group delimiter;
  %global cond_name ppsa_name eus_name ftype_name species_group_name;
  

  %global evalid_group;
  %global cond_attribute_connection ppsa_cond_connection ppsa_eus_connection condprop macrprop micrprop subpprop 
          plot_var tpa;

  %if not(%symexist(max_row_group_num)) %then %do;
     %global max_row_group_num;
     %let max_row_group_num=1;
  %end;
  %if not(%symexist(max_col_group_num)) %then %do;
     %global max_col_group_num;
     %let max_col_group_num=1;
  %end;
  %if not(%symexist(title_height)) %then %do;
     %global title_height;
     %let title_height= ;
  %end;
  %if not(%symexist(footnote)) %then %do;
     %global footnote footnote1 footnote2 footnote3;
     %global footnote_height footnote_height1 footnote_height2 footnote_height3;
  %end;


  %if not(%symexist(delete_empty_row)) %then %do;
     %global delete_empty_row delete_empty_col;
     %let delete_empty_row=1 ;
     %let delete_empty_col=1 ;
  %end;

  %if not(%symexist(max_replace)) %then %do;
     %global max_replace;
     %let max_replace=0 ;
  %end;

  %if not(%symexist(col_break_tabadd0)) %then %do;
     %global col_break_tabadd0 ;
     %let col_break_tabadd0=;
  %end;
  

  %global copy_x_y_rest ;
  %do i=1 %to 500;
    %global variable&i.x variable&i.y variable_desc&i. col_rest&i.x col_rest&i.y col_description&i.
            row_rest&i.x row_rest&i.y row_description&i. col_width&i. col_width_se&i.;
    %global rscd&i. evalid&i. evaldesc&i. s_evalid&i. s_evalid_add&i.;
	%global row_format&i.;
	%global replace_from&i. replace_to&i.;

    %if not(%symexist(col_var&i.x)) %then %do;
       %global col_var&i.x;
       %let col_var&i.x=1;
	%end;
    %if not(%symexist(col_var&i.y)) %then %do;
       %global col_var&i.y;
       %let col_var&i.y=1;
	%end;
    %if not(%symexist(row_var&i.x)) %then %do;
       %global row_var&i.x;
       %let row_var&i.x=1;
	%end;
    %if not(%symexist(row_var&i.y)) %then %do;
       %global row_var&i.y;
       %let row_var&i.y=1;
	%end;
    %if not(%symexist(adj_factor&i)) %then %do;
       %global adj_factor&i;
       %let adj_factor&i=1;
	%end;
    %if not(%symexist(col_group&i.)) %then %do;
       %global col_group&i. col_format&i. col_foot&i.;
       %let col_group&i.=1;
	%end;

    %if not(%symexist(col_break&i.)) %then %do;
       %global col_break&i. col_break_tabadd&i. col_break_title&i.;
       %let col_break&i.=0;
	%end;

    %if not(%symexist(col_width&i.)) %then %do;
       %global col_width&i. col_width_se&i.;
       %let col_width&i.=;
       %let col_width_se&i.=;
	%end;
    %if not(%symexist(row_height&i.)) %then %do;
       %global row_height&i.;
       %let row_height&i.=;
	%end;
    %if not(%symexist(print_title&i.)) %then %do;
       %global print_title&i.;
	   %let print_title&i. =;
	%end;
/*
    %do j=1 %to 5;
      %global col_group&i.v&j. col_format&i.v&j. row_group&i.v&j. row_format&i.v&j.;
    %end;
*/
    %global extra_row&i. extra_row_note&i.;
  %end;
  %do i=1 %to 500;
    %global col_group_name&i. row_group_name&i. row_group&i. col_group&i. col_group_foot&i. empty_row_group&i.;
	%let empty_row_group&i.=;
  %end;

/*
  %do i=1 %to 300;
    %global ;
  %end;
*/
%mend;


%define_global_macros;



%macro validate_datasource;

%if (not(%sysfunc(exist(&cond.)) and %sysfunc(exist(&ppsa.)) and %sysfunc(exist(&eus.)) and
         %sysfunc(exist(&attribute_tablex.)) and %sysfunc(exist(&ftype.)) and %sysfunc(exist(&species_group.)) ))
     %then %do;
       %display define_datasource.datasource blank;
       %if &datasource. = 1 or &datasource. = 2  %then %do;
          %display define_datasource.nims;
          %let use_idb=;
          %end;
       %else %if &datasource. = 3 %then %do;
          %display define_datasource.access;
          %let use_idb=;
          %end;
       %else %if &datasource. = 4 %then %do;
          %display define_datasource.sas;
          %let use_idb=;
          %end;
       %else %if &datasource. = 5 %then %do;
          %display define_datasource.idb;
          %let use_idb=1;
          %let sas_library=work;
          %end;
       %all_one_datasource;
       %display please_wait.data noinput blank;
       %get_data;
%end;

%mend;


%macro reset_excel_format;
%let paper_size = 1;
%let evaldescr_yes=2;
%let number_of_digits=0;
%let footnote= ;
%let footnote_height= ;
%let footnote1= ;
%let footnote_height1= ;
%let footnote2= ;
%let footnote_height2= ;
%let footnote3= ;
%let footnote_height3= ;
%let rounding_threshold = 0 ;
%let delete_empty_row=1 ;
%let delete_empty_col=1 ;
%let title=;
%let title_height= ;

%let rowname_width= ;
%let rowname2_width= ;

%let num_col_break=0 ;

%let add_tablex= ;
%let add_tabley= ;
%let add_connectionx= ;
%let add_connectiony= ;


%let max_replace=0 ;

/*
%let orange_value = 3;
%let red_value = 4;
%let color_large_value = 2;
*/
%do i=1 %to 500;
    %let col_var&i.x=1;
    %let col_var&i.y=1;
    %let row_var&i.x=1;
    %let row_var&i.y=1;
    %let col_group&i.=1;
    %let row_group&i.=1;
    %let extra_row&i.=;
	%let row_format&i.=;
    %let row_height&i.=;
    %let col_width&i.=;
    %let col_width_se&i.=;
    %let col_foot&i.=;
	%let col_width&i.= ;
    %let col_width_se&i.= ;
    %let col_break&i.= ;
    %let col_break_tabadd&i.= ;
    %let col_break_title&i.= ;

    %let variable&i.x=;
    %let variable&i.y=;
    %let variable_desc&i.=;
    %let col_rest&i.x=;
    %let col_rest&i.y=;
    %let col_description&i.=;
    %let row_rest&i.x=;
    %let row_rest&i.y=;
    %let row_description&i.=;

/*
    %let evalid&i.=;
    %let evaldesc&i.=;
    %let s_evalid&i.=;
	%let s_evalid_add&i.=;
*/
    %let replace_from&i.=;
    %let replace_to&i.=;

  %end;
  %do i=1 %to 150;
    %let col_group_name&i.=;
    %let row_group_name&i.=;
	%let col_group_foot&i.= ;
  %end;
%mend;



%macro available_evalid;
%if  "&use_idb." ne "1" %then %do;
  proc sql ;
    create table temp_evalid as
      select distinct eus.rscd, eus.evalid
      from eus_access eus
	  order by eus.rscd, eus.evalid
      ;
  quit;

  data temp2_evalid;
    set eus_access(keep = rscd evalid evaldescr);
  run;

  proc sort data = temp2_evalid;
    by rscd evalid;
  run;

  data temp_evalid;
    merge temp_evalid temp2_evalid;
    by rscd evalid;
    if first.evalid then output;
  run;

  proc sql noprint;
    select distinct eus.rscd, eus.evalid, eus.evaldescr
      into :avail_rscd separated by '|', :avail_evalid separated by '|', :avail_evaldesc separated by '|'
      from temp_evalid eus
      ;
  quit;

  %let num_avail_evalid=&sqlobs.;

  %let start=8;
  %let evalid_def_temp=;
  %let evalid_desc_temp=;
  %do i = 1 %to &num_avail_evalid.;
    %let rscd&i.=%scan(%bquote(&avail_rscd.), &i, '|');
    %let evalid&i.=%scan(%bquote(&avail_evalid.), &i, '|');
    %let evaldesc&i.=%scan(%bquote(&avail_evaldesc.), &i, '|');
	%let print_title&i. = &&evaldesc&i. ;

    %let evalid_def_temp=&evalid_def_temp.  #%eval(&start.+&i.) @13 s_evalid_add&i. 1 attr=underline @20 s_evalid&i. 1 attr=underline #%eval(&start.+&i.) @29 "&&evalid&i.";
    %let evalid_desc_temp=&evalid_desc_temp. #%eval(&start.+&i.) @36 print_title&i. 80 attr=underline;

  %end;


  %window choose_evalid menu=proclib.menucat.project color=white
    #3 @12 'Which inventories do you want to evaluate?'
    // @12 'Under "Select", enter the same number/letter (i.e. 1) to combine different inventories.'
     / @12 'Enter different number/letters to obtain seperate estimates for different inventories.'
	// @12 '+/-'
     @18 'Select'
     @28 'Evalid'
     @41 'Description'
	 
    &evalid_def_temp.
    &evalid_desc_temp.

    ///// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";
%end;
%else %do;
  %window choose_evalid menu=proclib.menucat.project color=white
    #3 @15 'Which inventories do you want to evaluate?'
    // @20 'Enter the same number (i.e. 1) to combine different inventories.'
    // @20 'Do NOT combine Oregons area and volume stratification.'
    //// @20 'Select'
     @30 'Description'
    // @20 s_evalid1 1 attr=underline @30 "California (do not use for woodland species volume estimates)"
    // @20 s_evalid2 1 attr=underline @30 "California FIA woodland species only. Do not combine with any other inventory."
    // @20 s_evalid3 1 attr=underline @30 "Oregon (Area Stratification)"
    // @20 s_evalid4 1 attr=underline @30 "Oregon (Volume Stratification)"
    // @20 s_evalid5 1 attr=underline @30 "Washington "
    // @20 s_evalid6 1 attr=underline @30 "Washington 2000-2001 (Area Stratification)"
    // @20 s_evalid7 1 attr=underline @30 "Washington 2000-2001 (Volume Stratification)"

    ///// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

%let num_avail_evalid=5;
%end;
%mend;


%macro group_evalid;
%if  "&use_idb." ne "1" %then %do;

  proc sql noprint;
    create table temp_group_evalid as
      select distinct eus.rscd, eus.evalid, eus.evaldescr
        from eus_access eus
      ;
  quit;
  %let num_avail_evalid=&sqlobs.;

  data temp_group_evalid (drop=i);
    set temp_group_evalid;
    /*do i = 1 to &num_avail_evalid.;*/
       %do i = 1 %to &num_avail_evalid.;
         if _n_ = &i. then do;
            same_group = "&&s_evalid&i";
			if "&&s_evalid_add&i." = "-" then subtract = evalid;
			evaldescr = "&&print_title&i.";
		 end;
       %end;
     /*end;*/
   run;
   %let subtract=.;
    proc sql noprint;
      select distinct evalid
        into :subtract separated by ','
        from temp_group_evalid
		  where subtract is not NULL;
        ;
    quit;

   proc sort data = temp_group_evalid;
     by same_group;
   run;
/*
 %if %sysfunc(exist(group_evalid)) = 0 %then %do;
*/
   data group_evalid(keep = same_group group_evaldescr);
     set temp_group_evalid;
       group_evaldescr = evaldescr;
   run;
/*
 %end;
*/
   proc sql noprint;
     create table group_names as
       select distinct same_group, group_evaldescr as old_group_evaldescr
         from group_evalid
         group by same_group
         ;
    quit;

   data group_evalid(keep = evalid_group evalid evaldescr group_evaldescr same_group);
     merge temp_group_evalid group_names;
     by same_group;

     length group_evaldescr $200.;
     retain last_group "-1" group 0 evalid_group . group_evaldescr "";

     if same_group = "" then delete;
     else if same_group ne last_group then do;
       last_group = same_group;
       group = group +1;
       evalid_group = group;
       group_evaldescr=old_group_evaldescr;
     end;
     else do;
       %let sum_across_evalid =2;
     end;
   run;

   proc sort data = group_evalid;
     by evalid_group;
   run;

   proc means data = group_evalid noprint;
     var evalid_group;
     by evalid_group;
     output out=same_group_count (drop = _type_ _freq_) n=n;
   run;

   data group_evalid;
     merge group_evalid same_group_count;
       by evalid_group;
    run;


    data temp(drop=evalid);
      set group_evalid ;
      by evalid_group;
      length ind1-ind30 $200.;
      retain num 0 ind1-ind30
                   evalid1-evalid30;
      if first.evalid_group then num = 0;
      num=num+1;

      array ind{*} ind1-ind30;
      array e{*}   evalid1-evalid30;

        do i=1 to n;
          if i=num then do;
              ind{i}=evaldescr;
              e{i} = evalid;
              end;
        end;
      if last.evalid_group then output;
    run;

   data temp;
    set temp;
    by evalid_group;
    array ind{*} ind1-ind30;

    window change_name menu=proclib.menucat.project color=white
        group = title      #3 @5 'Current inventories that you grouped into one group:' / ''
        group = ind        / @5 ind{i} persist=yes
        group = group_name /// @5 'Please change the group description name:'
                           // @5 group_evaldescr attr=underline required=yes
                           /// @15 'Do NOT use F3 for this window.';
    if n>1 then do;
     if first.evalid_group then
        display change_name.title blank noinput;
     do i=1 to n;
       display change_name.ind noinput;
     end;
     display change_name.group_name;
    end;
    run;

    data group_evalid(keep = evalid group_evaldescr evaldescr evalid_group same_group);
      set temp;
    array ind{*} ind1-ind30;
    array e{*}   evalid1-evalid30;

      do i= 1 to n;
        evaldescr=ind{i};
        evalid=e{i};
        output;
      end;
    run;

    %let evalid=;
    %let evaldescr=;

    proc sql noprint;
      select distinct evalid, evaldescr
        into :evalid separated by ',', :evaldesc separated by '|'
        from group_evalid
        ;
    quit;

    proc sql noprint;
      select distinct group_evaldescr, evalid_group
        into :group_evaldescr separated by '|',
             :evalid_group separated by ','
        from group_evalid
        ;
    quit;
%end;
%else %do;

  data group_evalid;

  %do i=1 %to 7;
    %if "&&s_evalid&i" ne "" %then %do;
      same_group = &&s_evalid&i;

      %if &i = 1 %then %do; 
          evaldescr = "California                        ";  

		  %if (&s_evalid1 = &s_evalid2 or &s_evalid1 = &s_evalid3) and &s_evalid1 = &s_evalid4 %then %do; 
	         group_evaldescr = "California, Oregon and Washington ";
		  %end;
		  %else %if &s_evalid1 = &s_evalid2 or &s_evalid1 = &s_evalid3 %then %do; 
	         group_evaldescr = "California and Oregon             ";
		  %end;
		  %else %if &s_evalid1 = &s_evalid4 %then %do; 
	         group_evaldescr = "California and Washington         ";
		  %end;
		  %else %do; 
	         group_evaldescr = "California                        ";
		  %end;
          evalid = 1; output;
          evalid = 2; output;
          evalid = 7; output;
/*          evalid = 11; output;*/
          evalid = 14; output;

      %end;
      %if &i = 3 %then %do; 
          evaldescr = "Oregon (Area Stratification)      "; 

		  %if &s_evalid1 = &s_evalid2 and &s_evalid1 = &s_evalid4 %then %do; 
	         group_evaldescr = "California, Oregon and Washington ";
		  %end;
		  %else %if &s_evalid1 = &s_evalid2 %then %do; 
	         group_evaldescr = "California and Oregon             ";
		  %end;
		  %else %if &s_evalid2 = &s_evalid4 %then %do; 
	         group_evaldescr = "Oregon and Washington             ";
		  %end;
		  %else %do; 
	         group_evaldescr = "Oregon                            ";
		  %end;
		  evalid = 3; output;
		  evalid = 5; output;
		  evalid = 7; output;
		  evalid = 8; output;
		  evalid = 12; output;

      %end;
      %if &i = 4 %then %do; 
          evaldescr = "Oregon (Volume Stratification)    "; 

		  %if &s_evalid1 = &s_evalid3 and &s_evalid1 = &s_evalid4 %then %do; 
	         group_evaldescr = "California, Oregon and Washington ";
		  %end;
		  %else %if &s_evalid1 = &s_evalid3 %then %do; 
	         group_evaldescr = "California and Oregon             ";
		  %end;
		  %else %if &s_evalid3 = &s_evalid4 %then %do; 
	         group_evaldescr = "Oregon and Washington             ";
		  %end;
		  %else %do; 
	         group_evaldescr = "Oregon                            ";
		  %end;
		  evalid = 9; output;
		  evalid = 10; output;
		  evalid = 7; output;
		  evalid = 8; output;
		  evalid = 12; output;

      %end;
      %if &i = 5 %then %do; 
          evaldescr = "Washington                        ";

		  %if (&s_evalid1 = &s_evalid2 or &s_evalid1 = &s_evalid3) and &s_evalid1 = &s_evalid4 %then %do; 
	         group_evaldescr = "California, Oregon and Washington ";
		  %end;
		  %else %if &s_evalid4 = &s_evalid2 or &s_evalid4 = &s_evalid3 %then %do; 
	         group_evaldescr = "Oregon and Washington             ";
		  %end;
		  %else %if &s_evalid1 = &s_evalid4 %then %do; 
	         group_evaldescr = "California and Washington         ";
		  %end;
		  %else %do; 
	         group_evaldescr = "Washington                        ";
		  %end;
	      evalid = 4; output;
	      evalid = 6; output;
	      evalid = 7; output;
	      evalid = 13; output;
      %end;
      %if &i = 6 %then %do; 
          evaldescr       = "Washington 2000-2001              ";
          group_evaldescr = "Washington 2000-2001              ";
	      evalid = 20; output;
	      evalid = 22; output;
      %end;
      %if &i = 7 %then %do; 
          evaldescr       = "Washington 2000-2001              ";
          group_evaldescr = "Washington 2000-2001              ";
	      evalid = 21; output;
	      evalid = 23; output;
      %end;

    %end;
  %end;
  run;

  proc sort data = group_evalid;
     by same_group evalid;
  run;

  data group_evalid;
     set group_evalid;
     by same_group;
	 if lag(evalid) = evalid and not first.same_group then delete;
  run;

  data group_evalid;
     set group_evalid;
     by same_group;
     retain evalid_group 0;
     if first.same_group then evalid_group = evalid_group +1;
  run;

  proc sql noprint;
      select distinct evalid
        into :evalid separated by ','
        from group_evalid
        ;
    quit;
/*
  proc sql noprint;
      select distinct evalid, evaldescr
        into :evalid separated by ',', :evaldesc separated by '|'
        from group_evalid
        ;
    quit;
*/
    proc sql noprint;
      select distinct group_evaldescr, evalid_group
        into :group_evaldescr separated by '|',
             :evalid_group separated by ','
        from group_evalid
        ;
    quit;
%end;
%mend;

*%group_evalid;


%macro import_def(file);
%reset_excel_format;

%include "&file." /LRECL= 5000;

%mend;
* %import_def(&import_file_location.);

%macro all_one_datasource;
 %if &datasource.=1 %then %do;
   %let cond_source=1;
   %let ppsa_source=1;
   %let eus_source=1;
   %let attribute_sourcex=1;
   %let ftype_source=1;
   %let species_group_source=1;
 %end;
 %else %if &datasource.=2 %then %do;
   %let cond_source=2;
   %let ppsa_source=2;
   %let eus_source=2;
   %let attribute_sourcex=2;
   %let ftype_source=2;
   %let species_group_source=2;
 %end;
 %else %if &datasource.=3 %then %do;
   %let cond_source=3;
   %let ppsa_source=3;
   %let eus_source=3;
   %let attribute_sourcex=3;
   %let ftype_source=3;
   %let species_group_source=3;
 %end;
 %else %if &datasource.=4 %then %do;
   %let cond_source=4;
   %let ppsa_source=4;
   %let eus_source=4;
   %let attribute_sourcex=4;
   %let ftype_source=4;
   %let species_group_source=4;
 %end;
 %else %if &datasource.=5 %then %do;
   %let cond_source=3;
   %let ppsa_source=4;
   %let eus_source=4;
   %let attribute_sourcex=3;
   %let ftype_source=3;
   %let species_group_source=3;
 %end;

%mend;




%macro different_datasource_window;
%window define_different_datasource menu=proclib.menucat.project color=white
       group=datasource
       #5  @20 'What are your datasources?'
       //  @20 nims_production_sel 1 attr=underline @22 'Oracle Server (NIMS Production Tables)'
       //  @20 nims_load_sel 1 attr=underline @22 'Oracle Server (NIMS Load Tables)'
       // @20 access_sel 1 attr=underline @22 'Access Tables'
       // @20 sas_sel 1 attr=underline @22 'SAS Library'
       // @20 use_idb 1 attr=underline @22 'IDB (experimental)'
    /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue."

       group=nims
       // @20 'Enter NIMS schema:'
       / @20 nims_schema 30 attr=underline
       // @20 'NIMS user name:'
       / @20 nims_user 30 attr=underline
       // @20 'NIMS user password:'
       / @20 nims_password 30 attr=underline

       group=access
       // @20 'Enter directory where the ACCESS file is located'
       // @20 access_dir 100 attr=underline

       group=sas
       // @20 'Enter SAS library'
       // @20 sas_library 100 attr=underline

       group=idb
       // @20 'Enter directory where the IDB ACCESS file is located'
       // @20 idb_dir 100 attr=underline

       group=not_exist
       // @20 "Directory &access_dir. does not contain an Access database."
	   /  @20 "Please enter a valid directory."


	   ;
%mend;


%macro different_datasource;
%different_datasource_window;
/*
%if "&use_idb." ne "" %then %let use_idb=1;
%if "&nims_production_sel." ne "" %then %let nims_production_sel=1;
%if "&nims_load_sel." ne "" %then %let nims_load_sel=1;
%if "&access_sel." ne "" %then %let access_sel=1;
%if "&sas_sel." ne "" %then %let sas_sel=1;
*/
%display define_different_datasource.datasource blank;

%if  "&nims_production_sel." ne "" or "&nims_load_sel." ne "" %then
   %display define_different_datasource.nims ;
%if  "&sas_sel." ne "" %then
   %display define_different_datasource.sas;
%if  "&access_sel." ne "" %then %do;
   %let num_ACCESS_files=-1;
   %do %until (&num_ACCESS_files.>0);
     %if &num_ACCESS_files=-1 %then %do;
       %display define_different_datasource.access ;
	 %end;
	 %else %do;
	   %different_datasource_window;
       %display define_different_datasource.not_exist blank noinput ;
       %display define_different_datasource.access ;
	 %end;

     %dir_list(&access_dir.,dir_access_old,mdb,"",ACCESS);
     %dir_list(&access_dir.,dir_access_new,accdb,"",ACCESS);
	 data dir_access;
	    set dir_access_old dir_access_new;
	 run;
    %dir_list_num(dir_access,ACCESS);

     proc sql noprint;
	   create table _null_ as
       select file
	     from dir_access
		   ;
     quit;
     %let num_ACCESS_files=&sqlobs.;
     %create_dir_list_sel_one(&access_dir.,dir_access,ACCESS);
   %end;
   %display display_file_list_sel_one;

   proc sql noprint;
     select "&access_dir.\" || file
	   into :access_file
	   from dir_access
	     where id = &select_ACCESS.
		 ;
	quit;
	%let access_file=%left(%trim(&access_file.));
   proc sql noprint;
     select file
	   into :access_file_name
	   from dir_access
	     where id = &select_ACCESS.
		 ;
	quit;
	%let access_file_name=%left(%trim(&access_file_name.));

%end;
   
%if  "&use_idb." ne "" %then %do;
   %let num_IDB_files=-1;
   %do %until (&num_IDB_files.>0);
     %if &num_IDB_files=-1 %then %do;
       %display define_different_datasource.idb ;
	 %end;
	 %else %do;
	   %different_datasource_window;
       %display define_different_datasource.not_exist blank noinput ;
       %display define_different_datasource.idb ;
	 %end;
     %dir_list(&idb_dir.,dir_idb_old,mdb,"",ACCESS);
     %dir_list(&idb_dir.,dir_idb_new,accdb,"",ACCESS);
	 data dir_idb;
	    set dir_idb_old dir_idb_new;
	 run;
    %dir_list_num(dir_idb,ACCESS);

     proc sql noprint;
	   create table _null_ as
       select file
	     from dir_idb
		   ;
     quit;
     %let num_IDB_files=&sqlobs.;
     %create_dir_list_sel_one(&idb_dir.,dir_idb,IDB);
   %end;
   %display display_file_list_sel_one;

   proc sql noprint;
     select "&idb_dir.\" || file
	   into :idb_file
	   from dir_idb
	     where id = &select_IDB.
		 ;
	quit;
	%let idb_file=%left(%trim(&idb_file.));

    %let attribute_sourcex=5;
    %let cond_source=4;
    %let ppsa_source=4;
    %let eus_source=4;
   
	%if "&attribute_namex." = "" %then %do;
       %let attribute_namex=tree_live;
	%end;
	%if "&attribute_namey." = "" %then %do;
       %let attribute_namey=tree_live;
    %end;
    %let cond_name=cond;
    %let ppsa_name=ppsa;
    %let eus_name=eus;
   %end;

%mend;


%macro datasource_detail;



%window different_datasource_detail menu=proclib.menucat.project color=white
   group=begin
   #4 @2 'Enter datasource for each table.'
   / @2 'You can change the table name if it is different than the default.'
   group=nims_load
   // @2 '1 for NIMS Load table'
   group=nims_prod
   / @2 '2 for NIMS Production table'
   group=access
   / @2 '3 for ACCESS tables'
   group=sas
   / @2 '4 for SAS library'
   group=idb
   / @2 '5 for IDB tables'
   group=input
  /// @2 attribute_sourcex 1 attr=underline required=yes @4 "Attribute/Tree table" @56 attribute_namex 40 attr=underline
   // @2 attribute_sourcey 1 attr=underline required=yes @4 "Denominator Attribute table (Ratio estimates)" @56 attribute_namey 40 attr=underline
   // @2 cond_source 1 attr=underline required=yes @4 "Condition table" @56 cond_name 40 attr=underline

   group=nims_40
  /// @2 "Enter datasource for NIMS 4.0"
    / @2 ppsa_source 1 attr=underline required=yes @4 "PPSA table" @56 ppsa_name 40 attr=underline
   // @2 eus_source 1 attr=underline required=yes @4 "EUS table" @56 eus_name 40 attr=underline
   
   group=fiadb_40
  /// @2 "Enter datasource for FIADB 4.0"
    / @2 pop_plot_stratum_assgn_source 1 attr=underline required=yes @4 "Pop Plot Stratum Assign table" @56 pop_plot_stratum_assgn_name 40 attr=underline
   // @2 pop_stratum_source 1 attr=underline required=yes @4 "POP Stratum table" @56 pop_stratum_name 40 attr=underline
   // @2 pop_eval_source 1 attr=underline required=yes @4 "POP Eval table" @56 pop_eval_name 40 attr=underline
   // @2 pop_estn_unit_source 1 attr=underline required=yes @4 "POP Estimation Unit table" @56 pop_estn_unit_name 40 attr=underline

   group=other
  /// @2 dwm_cond_source 1 attr=underline required=no @4 "DWM condition table (leave empty if DWM not used)" @56 dwm_cond_name 40 attr=underline
   // @2 cwd_source 1 attr=underline required=no @4 "CWD transsect table" @56 cwd_name 40 attr=underline
   // @2 fwd_source 1 attr=underline required=no @4 "FWD transsect table" @56 fwd_name 40 attr=underline

  /// @2 ftype_source 1 attr=underline @4 "Forest type table" @56 ftype_name 40 attr=underline
   // @2 species_group_source 1 attr=underline @4 "Species group table" @56 species_group_name 40 attr=underline
       /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";
;
%display different_datasource_detail.begin noinput;
%if  "&nims_load_sel." ne "" %then
  %display different_datasource_detail.nims_load noinput;
%if  "&nims_production_sel." ne "" %then
  %display different_datasource_detail.nims_prod noinput;
%if  "&access_sel." ne "" %then
  %display different_datasource_detail.access noinput;
%if  "&sas_sel." ne "" %then
  %display different_datasource_detail.sas noinput;
%if  "&use_idb." ne "" %then %do;
  %display different_datasource_detail.sas noinput;
  %display different_datasource_detail.idb noinput;
  %end;
%display different_datasource_detail.input;

%if  "&datasource_nims_40." ne "" %then %do;
  %display different_datasource_detail.nims_40;   
%end;
%else %if  "&datasource_fiadb_40." ne "" %then %do;
  %display different_datasource_detail.fiadb_40;   
%end;

%display different_datasource_detail.other;

%mend;


%macro datasource_idb_detail;

%window datasource_idb_detail menu=proclib.menucat.project color=white
   group=begin
   /// @20 'Change the table name if it is different from the default.'
   group=input
   /// @20 "Attribute table" @44 attribute_idb_name 40 attr=underline
   //  @20 "Condition table" @44 cond_idb_name 40 attr=underline
/*   //  @20 "PPSA table" @44 ppsa_idb_name 40 attr=underline
   //  @20 "EUS table" @44 eus_idb_name 40 attr=underline
*/
   //  @20 "Forest type table" @44 ftype_idb_name 40 attr=underline
   //  @20 "Species group table" @44 species_group_idb_name 40 attr=underline

  /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";
;
%display datasource_idb_detail;


%mend;




%macro datasource_version;

%window datasource_version menu=proclib.menucat.project color=white
   group=error
   	 / @20 "You need to select exactly one of the datasource options."
   group=all
   /// @20 'What version is your data in?'
   /// @20 "NIMS 4.0" @30 datasource_nims_40 1 attr=underline
   //  @20 "FIADB 4.0" @30 datasource_fiadb_40 1 attr=underline

  /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue."
     ;

%mend;




%macro copy_x_to_y_restrictions;
  %let variable_directiony=&variable_directionx.;
  %let restrictionsy= &restrictionsx.;

  %do i=1 %to &max_col_num.;
    %let col_rest&i.y=&&col_rest&i.x;
	  %if &variable_directiony. = 1 %then %do;
        %let col_var&i.x=&&col_var&i.x;
	  %end;
  %end;

  %do i=1 %to &max_row_num.;
    %let row_rest&i.y=&&row_rest&i.x;
  	%if &variable_directiony. = 2 %then %do;
      %let row_var&i.y=&&row_var&i.x;
	%end;
  %end;
  %do i=1 %to &max_variable_of_interest.;
      %let variable&i.y=&&variable&i.x;
  %end;
%mend;



%window copy_x_y_rest menu=proclib.menucat.project color=white
       group = general
       // @2 'Denominator'
       // @2 'Do you want to copy all restrictions and definitions from the Numerator?'
       // @2 '1  Yes'
       // @2 '2  No'
       // @2 copy_x_y_rest 1 attr=underline required=yes

       /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";


%macro save_table_def(file);

%if "%upcase(%substr(&file., %length(&file.)-2,3))" ne "SAS" %then %do;
  %let file = &file..sas;
  %put &file;
%end;

%let overwrite_file=2;

%if %sysfunc(fileexist(&file.)) %then %do;
   %window file_exist menu=proclib.menucat.project color=white
       /// @20 "File &file. exist. Do you want to overwrite it?"
       // @20 '1  Yes'
       // @20 '2  No'
       // @20 overwrite_file 1 attr=underline required = yes
    /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";
   %display file_exist blank;
%end;
%if not %sysfunc(fileexist(&file.)) or &overwrite_file.=1 %then %do;

data _null_;
  file "&file. ";
  put '/*';
  put 'This table configuration file has been created by tableing version ' "&version.";
  put '*/';
  put;
  put '%let title=' "&title. "';';
  put '%let title_height=' "&title_height. "';';
  put '%let title2=' "&title2 "';';
  put '%let max_variable_of_interest='"&max_variable_of_interest. "';';

%if "&add_tablex." ne "" %then %do;
  put;
  put '%let add_tablex=' "&add_tablex. "';';
%end;
%if "&add_connectionx." ne "" %then %do;
  put;
  put '%let add_connectionx=' "&add_connectionx. "';';
%end;
%if "&add_tabley." ne "" %then %do;
  put;
  put '%let add_tabley=' "&add_tabley. "';';
%end;
%if "&add_connectiony." ne "" %then %do;
  put;
  put '%let add_connectiony=' "&add_connectiony. "';';
%end;

  %if &ratio_y_n. = 1 %then %let max_r=2;
  %else %let max_r=1;
  %do r=1 %to &max_r;
    %let x_y=%scan(&r_var., &r.);
    put '%let ' "variable_direction&x_y.=&&variable_direction&x_y. "';';
    put '%let ' "restrictions&x_y.=%bquote(&&restrictions&x_y.) "';';
    put;
  %end;

  put '%let num_col_break=' "&num_col_break "';';
  %do i=1 %to &num_col_break.;
     put '%let ' "col_break&i.=&&col_break&i. "';';
     put '%let ' "col_break_tabadd&i.=&&col_break_tabadd&i. "';';
     put '%let ' "col_break_title&i.=&&col_break_title&i. "';';
  %end;
put;

put '%let rowname_width=' "&rowname_width. "';';
put '%let rowname2_width=' "&rowname2_width. "';';
put;
put '%let delete_empty_row=' "&delete_empty_row. "';';
put '%let delete_empty_col=' "&delete_empty_col. "';';
put;
put '%let attribute_type=' "&attribute_type. "';';
put '%let adj_factor=' "&adj_factor. "';';
put;
  put '%let row_var_name_out='"&row_var_name_out. "';';
  put '%let col_var_name_out='"&col_var_name_out. "';';
put;
  put '%let col=0;';
put; 
  %do col=1 %to &max_col_num.;
    put '/*Next Column*/';
    put '%let col =%eval(&col.+1);';
    put '%let col_description&col.='"&&col_description&col. "';';
    put '%let col_group&col.='"&&col_group&col. "';';
    put '%let col_format&col.='"&&col_format&col. "';';
	if "&&col_width&col." ne "" then
    put '%let col_width&col.='"&&col_width&col. "';';
    put '%let col_width_se&col.='"&&col_width_se&col. "';';
    %if &ratio_y_n. = 1 %then %let max_r=2;
	%else %let max_r=1;
    %do r=1 %to &max_r;
      %let x_y=%scan(&r_var., &r.);
      put '%let col_rest&col.'"&x_y.=%bquote(&&col_rest&col.&x_y.) "';';
	  %if &&variable_direction&x_y. = 1 %then %do;
        put '%let col_var&col.'"&x_y.=&&col_var&col.&x_y. "';';
	  %end;
	%end;
  %end;
  put;
  put '%let max_col_num=&col.;';
put;

put '%let row=0;';
put; 
  %do row=1 %to &max_row_num.;
    put '/*Next Row*/';
    put '%let row =%eval(&row.+1);';
    put '%let row_description&row.='"&&row_description&row. "';';
    put '%let row_group&row.='"&&row_group&row. "';';
    put '%let row_format&row.='"&&row_format&row. "';';
    put '%let row_height&row.='"&&row_height&row. "';';
    put '%let extra_row&row.='"&&extra_row&row. "';';
    %if "&&extra_row&row." ne "" %then %do;
      put '%let extra_row_note&row.='"&&extra_row_note&row. "';';
	%end;

    %if &ratio_y_n. = 1 %then %let max_r=2;
	%else %let max_r=1;
    %do r=1 %to &max_r;
      %let x_y=%scan(&r_var., &r.);
      put '%let row_rest&row.'"&x_y.=%bquote(&&row_rest&row.&x_y.) "';';
  	  %if &&variable_direction&x_y.. = 2 %then %do;
        put '%let row_var&row.'"&x_y.=&&row_var&row.&x_y. "';';
	  %end;
	%end;
  %end;
  put;
  put '%let max_row_num=&row.;';

put;
  %do i=1 %to &max_variable_of_interest.;
      put '%let ' "variable_desc&i.=%bquote(&&variable_desc&i.) "';';
    %if &ratio_y_n. = 1 %then %let max_r=2;
	%else %let max_r=1;
    %do r=1 %to &max_r;
      %let x_y=%scan(&r_var., &r.);
      put '%let ' "variable&i.&x_y.=%bquote(&&variable&i.&x_y.) "';';
	%end;
  %end;
put;
  put '%let units=' "&units."';';
put;
  put '%let exclude_col=' "&exclude_col. "';';
  put '%let exclude_row=' "&exclude_row. "';';
put;
  put '%let backgroundcolor=' "&backgroundcolor. "';';
put;
  put '%let count_plots_non_zero=' "&count_plots_non_zero. "';';
put;
  put '%let rounding_threshold=' "&rounding_threshold. "';';
put;
  put '%let number_of_digits=' "&number_of_digits. "';';
put;
  put '%let rtf_location=' "&rtf_location. "';';
put;
  put '%let page_orientation=' "&page_orientation. "';';
put;
  put '%let fit_one_page=' "&fit_one_page. "';';
put;
  put '%let evaldescr_yes=' "&evaldescr_yes. "';';
put;
  put '%let paper_size=' "&paper_size. "';';
put;
  put '%let print_date=' "&print_date. "';';
put;
  put '%let excel_font=' "&excel_font. "';';
put;
  put '%let color_large_value=' "&color_large_value. "';';
put;
  put '%let red_value=' "&red_value. "';';
put;
  put '%let orange_value=' "&orange_value. "';';
 

put;
put '%let ' "max_row_group_num=&max_row_group_num."';';
%do i = 1 %to &max_row_group_num.;
    put '%let ' "row_group_name&i.=&&row_group_name&i."';';
%end;
put;

put '%let ' "max_col_group_num=&max_col_group_num. "';';
%do i = 1 %to &max_col_group_num.;
    put '%let ' "col_group_name&i.=&&col_group_name&i. "';';
%end;
put;

put '%let footnote=' "&footnote. "';';
put '%let footnote_height=' "&footnote_height. "';';
put '%let footnote1=' "&footnote1. "';';
put '%let footnote_height1=' "&footnote_height1. "';';
put '%let footnote2=' "&footnote2. "';';
put '%let footnote_height2=' "&footnote_height2. "';';
put '%let footnote3=' "&footnote3. "';';
put '%let footnote_height3=' "&footnote_height3. "';';

put '%let ' "ratio_y_n=&ratio_y_n. "';';
put;
run;
%end;
%mend;


%macro save_data_def(file);

%if "%upcase(%substr(&file., %length(&file.)-2,3))" ne "SAS" %then %do;
  %let file = &file..sas;
  %put &file;
%end;

%if %sysfunc(fileexist(&file.)) %then %do;
   %window file_exist menu=proclib.menucat.project color=white
       /// @20 "File &file. exist. Do you want to overwrite it?"
       // @20 '1  Yes'
       // @20 '2  No'
       // @20 overwrite_file 1 attr=underline required = yes
    /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";
   %display file_exist blank;
%end;
%if not %sysfunc(fileexist(&file.)) or &overwrite_file.=1 %then %do;

data _null_;

  file "&file.";
  put '%let nims_schema=' " &nims_schema. "';';
  put '%let nims_user=' " &nims_user. "';';
  put '%let nims_password=' " &nims_password. "';';
put;
  put '%let use_idb=' " &use_idb. "';';
put;
%if "&access_file." ne "" %then %do;
    put '%let access_file=' " &access_file. "';';
%end;
%if "&sas_library." ne "" %then %do;
    put '%let sas_library=' " &sas_library. "';';
%end;
%if "&idb_file." ne "" %then %do;
    put '%let idb_file=' " &idb_file. "';';
  %end;
put;
  put '%let cond=' "&cond. "';';
  put '%let cond_source=' "&cond_source. "';';
  put '%let cond_name=' "&cond_name. "';';
put;
  put '%let cwd=' "&cwd. "';';
  put '%let cwd_source=' "&cwd_source. "';';
  put '%let cwd_name=' "&cwd_name. "';';
put;
  put '%let attribute_tablex=' "&attribute_tablex. "';';
  put '%let attribute_sourcex=' "&attribute_sourcex. "';';
  put '%let attribute_namex=' "&attribute_namex. "';';
  put '%let attribute_tabley=' "&attribute_tabley. "';';
  put '%let attribute_sourcey=' "&attribute_sourcey. "';';
  put '%let attribute_namey=' "&attribute_namey. "';';
put;

put;
  put '%let datasource_nims_40=' "&datasource_nims_40. "';';
  put '%let datasource_fiadb_40=' "&datasource_fiadb_40. "';';
put;

%if  "&datasource_nims_40." ne "" %then %do;
  put '%let ppsa=' "&ppsa. "';';
  put '%let ppsa_source=' "&ppsa_source. "';';
  put '%let ppsa_name=' "&ppsa_name. "';';
put;
  put '%let eus=' "&eus. "';';
  put '%let eus_source=' "&eus_source. "';';
  put '%let eus_name=' "&eus_name. "';';
put;

%end;
%else %if  "&datasource_fiadb_40." ne "" %then %do;

  put '%let pop_plot_stratum_assgn=' "&pop_plot_stratum_assgn. "';';
  put '%let pop_plot_stratum_assgn_source=' "&pop_plot_stratum_assgn_source. "';';
  put '%let pop_plot_stratum_assgn_name=' "&pop_plot_stratum_assgn_name. "';';
put;
  put '%let pop_stratum=' "&pop_stratum. "';';
  put '%let pop_stratum_source=' "&pop_stratum_source. "';';
  put '%let pop_stratum_name=' "&pop_stratum_name. "';';
put;
  put '%let pop_eval=' "&pop_eval. "';';
  put '%let pop_eval_source=' "&pop_eval_source. "';';
  put '%let pop_eval_name=' "&pop_eval_name. "';';
put;
  put '%let pop_estn_unit=' "&pop_estn_unit. "';';
  put '%let pop_estn_unit_source=' "&pop_estn_unit_source. "';';
  put '%let pop_estn_unit_name=' "&pop_estn_unit_name. "';';
put;
  %if "&use_adj_db." eq "1" %then %do;
    put;
  	put '%let use_adj_db = ' "&use_adj_db. "';';
  %end;
put;
put;
%end;

  put '%let ftype=' "&ftype. "';';
  put '%let ftype_source=' "&ftype_source. "';';
  put '%let ftype_name=' "&ftype_name. "';';
put;
  put '%let species_group=' "&species_group. "';';
  put '%let species_group_source=' "&species_group_source. "';';
  put '%let species_group_name=' "&species_group_name. "';';
put;
  put '%let census_water=' "&census_water. "';';
  put '%let one_datasource=' "&one_datasource. "';';
put;
run;
%end;
%mend;


proc pmenu catalog=proclib.menucat;

   menu project;

      item 'File' menu=f;

      menu f;
         item 'End';


quit;


%window start menu=proclib.menucat.project color=white
        #5 @28 "Welcome to the OK Tabling Program, Version &version." attr=highlight
        /// @ 20 'You can load already defined tables or you can define new tables.'
        / @ 20 'What would you like to do?'
        // @20 "1  Define or EDIT DATA source."
        // @20 "2  Define or EDIT new TABLE FORMATS."
        // @20 "3  Format Excel output."
        // @20 "4  Produce tables."
        // @20 "5  IMPORT previously defined TABLES and DATA files."
        // @20 "6  SAVE current definitions (tables and data)."
        // @20 "7  Advanced options."
        // @20 "8  Run previously defined tables in a batch."
        // @20 "Q  Quit."
        // @20 main_menu 1 attr=underline
        // @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

%window import_which menu=proclib.menucat.project color=white
       #5 @20 'Do you want to import a table or data location definition?'
       #7 @20 '1  Table'
       #9 @20 '2  Data location definition'
       #11 @20 "Enter C to cancle"
       #13 @20 which_import 1 attr=underline
       #19 @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

%window import menu=proclib.menucat.project color=white
       #5 @20 'Enter the SAS file containing the table definitions'
       #7 @20 "Enter C to cancle"
       #9 @20 import_file_location 100 attr=underline
       #19 @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

%window import_data_access menu=proclib.menucat.project color=white
       #5 @20 'Enter the SAS file containing the data location definitions'
       #7 @20 "Enter C to cancle"
       #9 @20 import_data_file_location 100 attr=underline
       #19 @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

%window save_table_def menu=proclib.menucat.project color=white
       #5 @20 'Enter the SAS file to save the table definitions'
       #7 @20 "Enter C to cancle"
       #9 @20 save_file_location 100 attr=underline
       #19 @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

%window save_data_def menu=proclib.menucat.project color=white
       #5 @20 'Enter the SAS file to save the data location definitions'
       #7 @20 "Enter C to cancle"
       #9 @20 save_data_file_location 100 attr=underline
       #19 @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

%window save_which_def menu=proclib.menucat.project color=white
       #5 @20 'Do you want to save table or data location definition?'
       #7 @20 '1  Table definition'
       #9 @20 '2  Data location definition'
       #11 @20 save_which 1 attr=underline required=yes
       #19 @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

%window define_table menu=proclib.menucat.project color=white
       group = general
       #3 @2 'Enter a Title for your Table'
       // @2 title 200 attr=underline

       //// @2 'Do you want to use any pre-defined rows/columns?'
       // @2 'You will be able to review them later.'
       // @2 '1  Yes'
       // @2 '2  No'
       // @2 pre_defined_row_col 1 attr=underline required=yes

       /// @2 'Do you want to estimate a ratio?'
       // @2 '1 Yes'
       // @2 '2 No'
       // @2 ratio_y_n 1 attr = underline required = yes

       /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

%window predefined menu=proclib.menucat.project color=white
       group=all
       #1 @20 'Predefined groups'
       group=row
       /// @20 'Choose one of the following pre-defined row groups:'
       / @20 '1  Species group'
       / @20 '2  Forest type group'
       / @20 '3  Forest types'
       / @20 '4  Owner group'
       / @20 '5  Stand size class'
       / @20 '6  Softwood/Hardwood'
       / @20 '7  Diameter class'
       / @20 'C  Cancle'
       / @20 predefined_row 1 attr=underline required=yes

       group=column
       /// @20 'Choose one of the following pre-defined column groups:'
       / @20 '1  Species group'
       / @20 '2  Forest type group'
       / @20 '3  Forest types'
       / @20 '4  Owner group'
       / @20 '5  Stand size class'
       / @20 '6  Softwood/Hardwood'
       / @20 '7  Diameter class'
       / @20 'C  Cancle'
       / @20 predefined_col 1 attr=underline required=yes
       group=restriction
       /// @20 'Choose one of the following pre-defined general restrictions:'
       / @20 '1  Forest land'
       / @20 '2  Timberland'
       / @20 '3  All live trees on forest land'
       / @20 '4  Growing stock on timberland'
       / @20 predefined_restriction 1 attr=underline required=yes
    /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";
       ;


%window additional_tables menu=proclib.menucat.project color=white
   group=begin
   #4 @2 'Do you need any additional tables (i.e., to link attribute and condition table)?'
   / @2 'You will usually not need any other tables.'
   / @2 'Leave blank if you don''t need any other tables.'
   /// @2 "Additional Numerator Tables"
   / @2 add_tablex 200 attr=underline
   // @2 "Connection settings for Numerator Tables"
   / @2 add_connectionx 200 attr=underline
   ///// @2 "Additional Denominator Tables"
   / @2 add_tabley 200 attr=underline 
   // @2 "Connection settings for Denominator Tables"
   / @2 add_connectiony 200 attr=underline
   /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue."
   ;


%window num_row_col menu=proclib.menucat.project color=white
       group=var
       #3 @2 'How many variables will you have?'
       // @42  max_variable_of_interest 3 attr=underline required=yes
       /// @20 'If you analyze tree data, enter TREE, for cwd data or any other, enter MIXED:'
	   // @42 attribute_type 10 attr=underline
       /// @2 'Enter the adjustment factor you want to use (only change if you are NOT analyzing tree or area)'
	   // @42 adj_factor 20 attr=underline

       group=row
       /// @2 'How many rows does your table have?'
       // @42  max_row_num 3 attr=underline required=yes

       group=col
       /// @2 'How many columns does your table have?'
       // @42  max_col_num 3 attr=underline required=yes

       //// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";


%window define_num_attribute_variable menu=proclib.menucat.project color=white
       #5 @20 'How many attribute/tree level variables do you want to analyze?'
       // @20 'For most tables you want only one variable.'
       // @20  max_variable_of_interest 1 attr=underline
       /// @20 'If you analyze tree data, enter TREE, for cwd data, enter CWD, for anyother, enter OTHER:'
	   // @20 attribute_type 10 attr=underline
       /// @2 'Enter the adjustment factor you want to use (only change if you are NOT analyzing tree or area)'
	   // @20 adj_factor 20 attr=underline
	   /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";


%window define_units menu=proclib.menucat.project color=white
       #5 @20 'What are the units you want to display (i.e., 1000)?'
       #7 @20  units 12 attr=underline
       #10 @20 'Enter a discription'
       #12 @20 title2 30 attr=underline
	   /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";


%window one_datasource menu=proclib.menucat.project color=white
       //// @20 'Is your data coming from one datasource?'
       // @20 '1  Yes'
       // @20 '2  No'
       // @20  one_datasource 1 attr=underline required=yes
       /// @20 'Is census water considered to be in the population (depends on your stratification?)'
       // @20 '1  Yes'
       // @20 '2  No'
       // @20  census_water 1 attr=underline required=yes
      /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";


%window census_water menu=proclib.menucat.project color=white
       /// @20 'Is census water considered to be in the population (depends on your stratification?)'
       // @20 '1  Yes'
       // @20 '2  No'
       // @20  census_water 1 attr=underline required=yes
    /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";


%window define_datasource menu=proclib.menucat.project color=white
       group=datasource
       #5  @20 'What is your datasource?'
       //  @20 '1  Oracle (NIMS Production Tables)'
       //  @20 '2  Oracle (NIMS Load Tables)'
       // @20 '3  Access Tables'
       // @20 '4  SAS Library'
       // @20 '5  IDB (experimental)'
       // @20  datasource 1 attr=underline required=yes

       group=nims
       /// @20 'Enter Oracle (NIMS) schema:'
       / @20 nims_schema 30 attr=underline
       // @20 'Oracle (NIMS) user name:'
       / @20 nims_user 30 attr=underline
       // @20 'Oracle (NIMS) user password:'
       / @20 nims_password 30 attr=underline

       group=access
       /// @20 'Enter Access file location'
       // @20 access_file 100 attr=underline

       group=sas
       /// @20 'Enter SAS library'
       // @20 sas_library 100 attr=underline

       group=idb
       /// @20 'Enter IDB access file'
       // @20 access_file 100 attr=underline
    /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

/*
%window import_datasource menu=proclib.menucat.project color=white
       #5 @20 'Import/assign data source now?'
       #7 @20 'Data import might take a few minutes.'
       #9 @20 '1  Yes'
       #11 @20 '2  No'
       #13 @20  import_now 1 attr=underline required=yes;
*/
%window eus_not_exist menu=proclib.menucat.project color=white
       #5 @20 'EUS table does not exist/not assigned.'
       #7 @20 'You need to import EUS table first before you can choose a evaluation'
       #9 @20 'Please select and import/assign data source first'
       #15 @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";


%window display_tables menu=proclib.menucat.project color=white
       group=general
       #5 @20 'Do you want to display or save the tables to a RTF file (Word) or Excel?'
       // @20 '1  Display in SAS output window'
       // @20 '2  Save as EXCEL'
       // @20  display_tables 1 attr=underline required=yes
	   group=excel
       /// @20 'File location and name for the Excel file:'
       // @20 'Directory: ' rtf_dir 120 attr=underline
       // @20 'File name: ' rtf_location 120 attr=underline
    /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";


%macro dir_location(sas_file_type);

%if not(%symexist(dir_&sas_file_type._location)) %then %do;
  %global dir_&sas_file_type._location;
  %let dir_&sas_file_type._location=&tabeling_dir.;
%end;

%if not(%symexist(pattern_&sas_file_type.)) %then %do;
  %global pattern_&sas_file_type.;
  %let pattern_&sas_file_type.=;
%end;


%window dir_location menu=proclib.menucat.project color=white
       group=general
       #5 @20 'Enter the directory where the SAS scripts are located:'
       // @20 dir_&sas_file_type._location 100 attr=underline
       group=pattern
       /// @20 'Enter any file name pattern for the SAS files:'
       // @20 pattern_&sas_file_type. 60 attr=underline
    /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";


%mend;

%window advanced_options menu=proclib.menucat.project color=white
/*       /// @20 'Do you want to exclude certain columns from being displayed?'
       // @20 exclude_col 90 attr=underline
       /// @20 'Do you want to exclude certain rows from being displayed?'
       // @20 exclude_row 90 attr=underline
*/
       /// @20 'Count number of plots with values larger zero.'
       / @20 count_plots_non_zero 1 attr=underline

       /// @20 'Close Excel sheets after producing tables?'
       / @20 '1  Yes'
       / @20 '2  No'
       / @20  close_excel_sheet 1 attr=underline


	   /// @20 'Do you want to color large absolute values (as compared to standard error)?'
       / @20 '1  Yes'
       / @20 '2  No'
       / @20 color_large_value 1 attr=underline

	   /// @20 'Color orange values larger than x times the standard error:'
       / @20  orange_value 6 attr=underline

	   /// @20 'Color red values larger than x times the standard error:'
       / @20  red_value 6 attr=underline
     //// @20 "Use FIADB adjustment factor from Database" @70 use_adj_db 1 attr= underline
     //// @20 "Use condition adjustment factor for tree adjustments" @80 use_cond_adj 1 attr= underline

    /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

/*       /// @20 'Are you Glenn Christensen?'
       // @20 glenn 1 attr=underline;
*/

%window please_wait menu=proclib.menucat.project color=white
       group=data
       #6 @30 'Please wait.'
       #9 @16 'It may take some time until the data is imported.'
       group=table
       #6 @30 'Please wait.'
       #9 @16 'It may take some time until the tables are produced.';



%macro enter_row_col(x_y);
%let start=14;
%if &ratio_y_n. eq 2 %then %do;
  %let var_display=;
%end;
%else %if "&x_y." = "x" %then %do;
  %let var_display=#1 @30 "Enter Numerator " ;
%end;
%else %do;
  %let var_display=#1 @30 "Enter Denominator " ;
%end;
%let var_display=&var_display. / @1 "Var";
%do i=1 %to &max_variable_of_interest.;
  %let var_display=&var_display. " &i.: &&variable&i.&x_y.  ";
%end;

%if &max_variable_of_interest. le 1 %then %let var_display=;

%if &&variable_direction&x_y.=2 %then %do;
  %let row_def = // @1 'Var'
       @6 'Enter row restrictions'
       @87 'Description';
    %let col_def = // @1 'Enter column restrictions'
         @82 'Description';
    %end;
  %else %do;
  %let col_def = // @1 'Var'
       @6 'Enter column restrictions'
       @87 'Description';
    %let row_def = // @1 'Enter row restrictions'
         @82 'Description';
  %end;



%do i = 1 %to &max_row_num.;
  %if &&variable_direction&x_y.=2  and &max_variable_of_interest. >1 %then %do;
    %let row_def=&row_def.  / @1 row_var&i.&x_y. 2 attr=underline required=yes
                              @6 row_rest&i.&x_y. 80 attr=underline
                              @87 row_description&i. 40 attr=underline;
  %end;
  %else %do;
    %let row_def=&row_def.  / @1 row_rest&i.&x_y. 80 attr=underline
                              @82 row_description&i. 40 attr=underline;
    %let row_var&i.&x_y. = 1;
  %end;
%end;

%do i = 1 %to &max_col_num.;

  %if &&variable_direction&x_y. ne 2 and &max_variable_of_interest. >1 %then %do;
    %let col_def=&col_def.  / @1 col_var&i.&x_y. 2 attr=underline required=yes
                              @6 col_rest&i.&x_y. 80 attr=underline
                              @87 col_description&i. 40 attr=underline;
  %end;
  %else %do;
    %let col_def=&col_def.  / @1 col_rest&i.&x_y. 80 attr=underline
                              @82 col_description&i. 40 attr=underline;
    %let col_var&i.&x_y. = 1;
  %end;
%end;

%window enter_row_col menu=proclib.menucat.project color=white
    GROUP=general
    &var_display.
    // @1 'Enter any general condition and/or tree level restrictions'
    // @1 restrictions&x_y. 200 attr=underline
    GROUP=row
    // @1 'Enter a row group name'
       @30 row_var_name_out 70 attr=underline
    &row_def.
    GROUP=col
    // @1 'Enter a column group name'
       @30 col_var_name_out 70 attr=underline
    &col_def.
    /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

%mend;




%macro enter_row_col_format;
%let start=14;
%if &variable_directionx.=2 %then %do;
   %let max_row_var=&max_variable_of_interest.;
   %let max_col_var=1;
   %end;
%else %do;
   %let max_row_var=1;
   %let max_col_var=&max_variable_of_interest.;
   %end;

%let row_def =;
%let row_des =;
%do i = 1 %to &max_row_num.;
    %let descr_temp=%qsubstr("&&row_description&i.",2,29);
    %let row_def=&row_def.  / @1 row_group&i. 2 attr=underline
	                          @7 extra_row&i. 1 attr=underline
                              @11 row_format&i. 6 attr=underline
                              @19 row_height&i. 5 attr= underline
                              @26 "&descr_temp.";
%end;

%let row_group_def=;
%if "&max_row_group_num." ne "" %then %do;
  %let row_group_def=@52 'Group' @60 'Row Group Name';
  %do i = 1 %to &max_row_group_num.;
     %let row_group_def=&row_group_def. / @54 "&i." @60 row_group_name&i. 80 attr=underline;
  %end;
%end;

  %let extra_print=0;
  %let extra_row_def=/ @54 'Ex.' @60 'Extra Row Insert' ;
  %do i = 1 %to &max_row_num.;
     %if "&&extra_row&i." ne "" %then %do;
       %let extra_row_def=&extra_row_def. / @54 "&i." @60 extra_row_note&i. 80 attr=underline;
	   %let extra_print=1;
	 %end;
  %end;
  %if &extra_print.=1 %then %do;
    %let row_group_def=&row_group_def. / &extra_row_def.;
  %end;

%let col_def =;
%let col_des =;
%do i = 1 %to &max_col_num.;
    %let col_def=&col_def.  / @1 col_group&i. 2 attr=underline
	                          @7 col_width&i. 5 attr=underline
	                          @13 col_width_se&i. 5 attr=underline
                              @23 "&&col_description&i.";
%end;

%let col_group_def=;
%if "&max_col_group_num." ne "" %then %do;
  %let col_group_def=@52 'Group' @60 'Column Group Name';
  %do i = 1 %to &max_col_group_num.;
     %let col_group_def=&col_group_def. / @54 "&i." @60 col_group_name&i. 80 attr=underline;
  %end;
%end;


%window enter_row_col_format menu=proclib.menucat.project color=white
    GROUP=general
    #1 @1  
	'Page print format:      1) Portrait, 2) Landscape ' page_orientation 1 attr=underline
	/  'Fit output on one page: 1) True,     2) False     ' fit_one_page 1 attr=underline

    / 'How many significant digits should the output display? ' number_of_digits 1 attr=underline
	/ 'What is the threshold for displaying estimates? ' rounding_threshold 10 attr=underline
    / 'Excel paper size:  1 Letter, 2 Legal '  paper_size 1 attr=underline
    / 'Display Evaldescr in table header?  1  Yes,  2  No  ' evaldescr_yes 1 attr=underline
	/ 'Font for Excel output: ' excel_font 50 attr=underline
	GROUP=row
    // 'Rows and columns can be grouped and formated.'
	/ 'Valid formats include: (g)ray, (b)old, (i)talic, (u)nderline, (t)ab, (d)elete, (l)ine above'
    // @4 'Row groups and formating'
    // @1 'Group'
	   @7 'Ex.'
       @11 'Format'
	   @19 'Height'
       @26 'Row Description'
	   


    &row_def.
    GROUP=col
    /// @4 'Column groups'
    // @1 'Group'
	   @7 'Width'
	   @13 'Width SE'
       @23 'Column Description'
    &col_def.
    GROUP=row_group
    #9 &row_group_def " "
    GROUP=col_group
    #%eval(&max_row_num.+14) &col_group_def " "
    #%eval(&max_row_num. + &max_col_num. +21) @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";
%mend;

%macro populate_format;
*%let last_group=&row_group1v1.;

%if &variable_directionx.=2 %then %do;
   %let max_row_var=&max_variable_of_interest.;
   %let max_col_var=1;
   %end;
%else %do;
   %let max_row_var=1;
   %let max_col_var=&max_variable_of_interest.;
   %end;

%let last_group=&row_group1.;
%let run=1;

%if "&last_group."="" %then %let last_group = &run.;
%do i = 1 %to &max_row_num.;
  %if "&&row_group&i." = "" %then %let row_group&i.=&last_group;
  %if &last_group ne &&row_group&i. %then %do;
       %let last_group = &&row_group&i.;
       %let run=%eval(&run. + 1);
  %end;
  %let row_group&i. = &run.;
%end;

%let max_row_group_num = &run.;

%let run=1;
%let last_group=&col_group1.;
%if "&last_group."="" %then %let last_group = &run.;
%do i = 1 %to &max_col_num.;
    %if "&&col_group&i." = "" %then %let col_group&i.=&last_group;
    %if &last_group ne &&col_group&i. %then %do;
         %let last_group = &&col_group&i.;
         %let run=%eval(&run. + 1);
      %end;
    %let col_group&i. = &run.;
%end;

%let max_col_group_num = &run;




%mend;



%macro enter_attribute_var(x_y);

%if &ratio_y_n. eq 2 %then %do;
  %let initial=/ @2  'Enter Variable of Interest';
%end;
%else %if "&x_y." = "x" %then %do;
  %let initial=/ @10 "Enter Variable of Interest for Numerator " ;
%end;
%else %do;
  %let initial=/ @10 "Enter Variable of Interest for Denominator " ;
%end;
%let initial=&initial // @2 'For area estimates enter keywork "AREA"' 
                       / @2 'To estimate number of trees, enter "1"';

%let var_def=/;
%do i = 1 %to &max_variable_of_interest.;
  %let var_def=&var_def. / @2 "&i." @4 variable&i.&x_y. 80 attr=underline;
  %let var_def=&var_def. @90 variable_desc&i. 40 attr=underline;
%end;

%let display_hor=;
%if &max_variable_of_interest. > 1 %then %do;
  %let display_hor=//// @20 'Do you want the variable displayed horizontally or vertically in the table?'
  // @20 '1 Horizontal'
  // @20 '2 Vertical'
  // @20 variable_direction&x_y. 1 attr=underline required=yes
  /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";
%end;
%else
  %let display_hor=/// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";
/*
%put &var_def.;
%put &var_des.;
%put &display_hor.;
*/
%window enter_attribute_var menu=proclib.menucat.project color=white
    group=variable_def
    &initial. 
	 // @4 'Variable'
       @90 'Description'
    &var_def.
    GROUP=variable_direction
    &display_hor.;
%mend;



%macro run_indy9_tables;


  %validate_datasource;

  %display display_tables.general blank;
  %if &display_tables. = 2 %then %do;
     %display display_tables.excel;
  %end;


  %available_evalid;
  %if &num_avail_evalid. > 0 %then %do;
    %display choose_evalid;
    %group_evalid;
/*
    %let evalid=;
    %let evaldescr=;

    proc sql noprint;
      select distinct evalid, evaldescr, group_evaldescr, evalid_group
        into :evalid separated by ',', :evaldesc separated by '|',
             :group_evaldescr separated by '|',
             :evalid_group separated by ','
        from group_evalid
        ;
    quit;
*/

    %display please_wait.table noinput blank;

    %do indy9=1 %to 9;
/*    %let rtf_location =&&base_outfile.&indy9..rtf;*/
      %include "&tabeling_dir.\indy9_table&indy9..sas" /LRECL= 5000;
      %run_programs;
    %end;
  /*%let rtf_location =&base_outfile..rtf;    */
  %end;
%mend;



%macro dir_list(dir,outfile,file_extension, pattern, file_type);

filename DIRLIST pipe "dir /B ""&dir.\*.&file_extension. """;

%let file_extension_length = %length(.&file_extension);

data &outfile. ;
     length file $1024 ;
     infile dirlist length=reclen ;
     input file $varying1024. reclen ;
	 if count(upcase(file),upcase(&pattern.)) = 0 then delete;
	 if upcase(substr(file,length(file)-&file_extension_length.+1,&file_extension_length.)) ne 
        upcase(".&file_extension.") then delete;
run ;

%mend;


%macro dir_list_num(outfile, file_type);

data &outfile.;
  set &outfile.;
  id = _n_;
run;

proc sql noprint;
  create table _null_ as
    select *
	  from &outfile.
	  ;
quit;

%if not(%symexist(num_&file_type._files)) %then %do;
   %global num_&file_type._files;
%end;
%let num_&file_type._files=&sqlobs.;
%mend;


%macro create_dir_list(dir,file_list,file_type,export=0);
%put &&num_&file_type._files;

%do i=1 %to &&num_&file_type._files;
  %if not(%symexist(file_id_&file_type._&i.)) %then %do;
     %global file_id_&file_type._&i.;
     %let file_id_&file_type._&i.=;
  %end;
  %if &export. ne 0 %then %do;
    %if not(%symexist(file_export_&file_type._&i.)) %then %do;
       %global file_export_&file_type._&i.;
       %let file_export_&file_type._&i.=;
    %end;
  %end;
%end;


proc sql noprint;
    select "/ @2 file_id_&file_type._" || compress(put(id,3.0))|| " 1 attr=underline @5 " || '"' 
           || substrn(left(trim(file)),0,50) || '"'
		   %if &export. ne 0 %then %do;
             || " @56 file_export_&file_type._" || compress(put(id,3.0))|| " 80 attr=underline" 
			  , current_rtf_location
		   %end; 
	  into :file_&file_type._1 - :file_&file_type._%eval(&&num_&file_type._files.)
		   %if &export. ne 0 %then %do;
		   , :file_export_&file_type._1 - :file_export_&file_type._%eval(&&num_&file_type._files.)
		   %end; 
	  from &file_list.
	  ;
quit;

%let file_display=;

%do i=1 %to &&num_&file_type._files.;
  %let file_display=&file_display. &&file_&file_type._&i.;
%end;

%if "&rtf_dir." = "" %then %let rtf_dir = &dir.;


%window display_file_list menu=proclib.menucat.project color=white
    GROUP=general
    // @2 "Mark &file_type. files that should be included in the batch list."
	/  @2 "Edit Export file and Path where necessary."
	// @2 "Output directory: " rtf_dir 80 attr=underline
	// @5 "SAS Script File" @ 56 "Export File"
    / &file_display.
    /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

%mend;


%macro create_dir_list_sel_one(dir,file_list,file_type);
%if not(%symexist(select_&file_type.)) %then %do;
   %global select_&file_type.;
   %let select_&file_type.=;
%end;

proc sql noprint;
    select "/ @20 " || '"' || compress(put(id,3.0)) || '"' || " @24 " || '"' 
           || left(trim(file)) || '"'
	  into :file_&file_type._1 - :file_&file_type._%eval(&&num_&file_type._files)
	  from &file_list.
	  ;
quit;

%let file_display=;

%do i=1 %to &&num_&file_type._files;
  %let file_display=&file_display. &&file_&file_type._&i.;
%end;


%window display_file_list_sel_one menu=proclib.menucat.project color=white
    GROUP=general
    // @20 "Select &file_type. file:"
    / &file_display.
	/ @20 select_&file_type. 2 attr=underline required=yes
    /// @29 "Press " "ENTER" attr=underline " or hit " "F3" attr=underline " to continue.";

%mend;



%macro find_export_file(dir,file_list,file_type);

data &file_list.(drop=d rtf);
   length fileloc current_rtf_location dummy infile $ 300;
   set &file_list. ; /* read instream data       */

   fileloc="&dir.\" || file;
  /* The INFILE statement closes the current file 
     and opens a new one if FILELOC changes value 
     when INFILE executes                        */
   infile dummy1 filevar=fileloc filename=dummy
          end=EOF truncover; 

  /* DONE set to 1 when last input record read  */
   do until(EOF);
  /* Read all input records from the currently  */
  /* opened input file, write to export_list       */
     input d $ rtf $12. d $1. current_rtf_location $300. ;
	 infile=dummy;
     if upcase(rtf) eq upcase("rtf_location") then do;
	   current_rtf_location = substrn(current_rtf_location ,0, length(current_rtf_location) - 1);
       output;
	 end;   end;
run;

data &file_list.;
  set &file_list.;
  id = _n_;
run;

proc sql noprint;
  create table _null_ as
    select *
	  from &file_list.
	  ;
quit;

%if not(%symexist(num_&file_type._files)) %then %do;
   %global num_&file_type._files;
%end;
%let num_&file_type._files=&sqlobs.;
%mend;


%macro run_batch_tables;

*  %validate_datasource;

  %dir_location(sas);
  %display dir_location.general blank;
  %display dir_location.pattern;
  %dir_list(&dir_sas_location.,dir_sas,sas,"&pattern_sas.",SAS);
  %dir_list_num(dir_sas,SAS);

  %find_export_file(&dir_sas_location.,dir_sas,SAS);

  %create_dir_list(&dir_sas_location.,dir_sas,SAS,export=1);

  %display display_file_list;

  %available_evalid;
  %if &num_avail_evalid. > 0 %then %do;
    %display choose_evalid;
    %group_evalid;
/*
    %let evalid=;
    %let evaldescr=;

    proc sql noprint;
      select distinct evalid, evaldescr, group_evaldescr, evalid_group
        into :evalid separated by ',', :evaldesc separated by '|',
             :group_evaldescr separated by '|',
             :evalid_group separated by ','
        from group_evalid
        ;
    quit;
*/
  %end;

  %display please_wait.table noinput blank;
  %let close_excel_sheet=1;

  %do run_batch=1 %to &num_sas_files.;
     %if "&&file_id_sas_&run_batch." ne "" %then %do;
       proc sql noprint;
         select file
	       into :sas_file
		     from dir_sas
		     where id = &run_batch.
		     ;
	   quit;
	   %reset_excel_format;
       %include "&dir_sas_location.\%left(%trim(&sas_file.))"  /LRECL= 5000;
	   %let rtf_location = &&file_export_sas_&run_batch.;
	   %let display_tables=2;
	   /*don't ever use adjustment factors from fiadb*/
/*	   %let use_adj_db = 0;*/
       %run_programs;
	 %end;
  %end;
%mend;


%macro tabeling_menu;

%do %until (%upcase("&main_menu.")=%upcase("Q"));
  %let main_menu=;
  %display start;
  %if &main_menu. = 5 %then %do;
    %display import_which;
    %if &which_import.=1 %then %do;
	   %dir_location(import_def_sas);
       %display dir_location.general blank;
       %display dir_location.pattern;
       %dir_list(&dir_import_def_sas_location.,dir_sas,sas,"&pattern_import_def_sas.",SAS_script);
       %dir_list_num(dir_sas,SAS_script);

       %create_dir_list_sel_one(&dir_import_def_sas_location.,dir_sas,SAS_script);
 
       %display display_file_list_sel_one;
       proc sql noprint;
         select "&dir_import_def_sas_location.\" || file
	       into :import_file_location
	         from dir_sas
	           where id = &select_SAS_script.
		       ;
          quit;
       %let import_file_location=%left(%trim(&import_file_location.));
       %import_def(&import_file_location.);
    %end;
    %else %if &which_import.=2 %then %do;
	   %dir_location(import_data_sas);
       %display dir_location.general blank;
       %display dir_location.pattern;
       %dir_list(&dir_import_data_sas_location.,dir_data_sas,sas,"&pattern_import_data_sas.",SAS_data);
       %dir_list_num(dir_data_sas,SAS_data);

       %create_dir_list_sel_one(&dir_import_data_sas_location.,dir_data_sas,SAS_data);
 
       %display display_file_list_sel_one;
       proc sql noprint;
         select "&dir_import_data_sas_location.\" || file
	       into :import_data_file_location
	         from dir_data_sas
	           where id = &select_SAS_data.
		       ;
          quit;
       %let import_data_file_location=%left(%trim(&import_data_file_location.));
	   %if "&import_data_file_location." ne "" %then %do;
	      %let use_adj_db =;
	   %end;
       %include "&import_data_file_location." /LRECL= 5000;

	   %display please_wait.data noinput blank;
       %get_data;

    %end;
  %end;
  %else %if &main_menu. = 2 %then %do;
     %display define_table.general blank;
        %if (&pre_defined_row_col. =1) %then %do;
		   %available_evalid; 
           %if &num_avail_evalid. > 0 %then %do;
              %display choose_evalid;
              %group_evalid;
			  /*
              %let evalid=;
              %let evaldescr=;

              proc sql noprint;
                select distinct evalid, evaldescr, group_evaldescr, evalid_group
                   into :evalid separated by ',', :evaldesc separated by '|',
                        :group_evaldescr separated by '|',
                        :evalid_group separated by ','
                   from group_evalid
                   ;
              quit;
			  */
              %end;
           %display predefined;
           %predefined;
              %display num_row_col.var blank;
		   %let pre_defined_row_col =2;
        %end;
        %else %do;
              %display num_row_col.var blank;
              %display num_row_col.row;
              %display num_row_col.col;
        %end;
		%display additional_tables;
        %if "&variable1x."="AREA" %then %do;
          %let max_variable_of_interest=1;
          %let variable1x=AREA;
          %let variable_desc1=;
          %enter_attribute_var(x);
          %display enter_attribute_var;
        %end;
        %else %do;
          %enter_attribute_var(x);
          %display enter_attribute_var;
          %do %while (not ("&variable_directionx." = "1" or "&variable_directionx." = "2"));
            %display enter_attribute_var.variable_direction blank;
          %end;
        %end;

        %display define_units;

        %enter_row_col(x);
        %display enter_row_col blank;
    /* %end;*/
     /*If  a ratio estimate
     */
     %if &ratio_y_n.=1 %then %do;
     /*ratio estimates*/
       %display copy_x_y_rest blank;
	   %if &copy_x_y_rest. = 1 %then %do; 
	     %copy_x_to_y_restrictions;
		 %let copy_x_y_rest = 2;
	   %end;
	   %enter_attribute_var(y);
       %display enter_attribute_var;
       %do %while (not ("&variable_directiony." = "1" or "&variable_directiony." = "2"));
           %display enter_attribute_var.variable_direction blank;
       %end;

       %enter_row_col(y);
       %display enter_row_col blank;

     %end;
  %end;
  %else %if &main_menu. = 1 %then %do;
  /*define data source*/
       %display census_water blank;
	   %datasource_version;
	   %display datasource_version.all;
		%do %while (("&datasource_nims_40." = "" and "&datasource_fiadb_40." = "") or ("&datasource_nims_40." ^= "" and "&datasource_fiadb_40." ^= ""));
			%display datasource_version blank;
		%end;

       %different_datasource;
       %datasource_detail;
       %display please_wait.data noinput blank;
       %get_data;
  %end;
  %else %if &main_menu. = 6 %then %do;
    /*save definitions*/
    %display save_which_def ;
    %if &save_which.=1 %then %do;
      %display save_table_def;
      %if (%upcase("&save_file_location.") ^= %upcase("C")) %then %do;
         %save_table_def(&save_file_location.);
        %end;
    %end;
    %else %if &save_which.=2 %then %do;
      %display save_data_def;
      %if (%upcase("&save_data_file_location.") ^= %upcase("C")) %then %do;
         %save_data_def(&save_data_file_location.);
        %end;
      %end;
  %end;
  %else %if &main_menu. = 3 %then %do;
      %enter_row_col_format;
      %display enter_row_col_format.general noinput blank;
      %display enter_row_col_format.row;
      %display enter_row_col_format.col;
      %populate_format;
      %enter_row_col_format;
      %display enter_row_col_format;
      %end;
  %else %if &main_menu. = 4 %then %do;
    %display display_tables.general blank;
	%if &display_tables. = 2 %then %do;
       %display display_tables.excel;
	%end;
    %available_evalid;
    %if &num_avail_evalid. > 0 %then %do;
      %display choose_evalid;
      %group_evalid;
	  /*
      %let evalid=;
      %let evaldescr=;

      proc sql noprint;
        select distinct evalid, evaldescr, group_evaldescr, evalid_group
          into :evalid separated by ',', :evaldesc separated by '|',
               :group_evaldescr separated by '|',
               :evalid_group separated by ','
          from group_evalid
          ;
      quit;
	  */


      %display please_wait.table noinput blank;
      %run_programs;
    %end;
    %else %do;
      %display eus_not_exist;
    %end;
  %end;
  %else %if &main_menu=7 %then %do;
    %display advanced_options blank;
    %if "&glenn." ne "" %then %do;
      %let backgroundcolor=orange;
    %end;
    %else %do;
      %let backgroundcolor=white;
    %end;
    %if "&count_plots_non_zero." ne "" %then %do;
      %let count_plots_non_zero=1;
    %end;
  %end;
  /*
  %else %if &main_menu=8 %then %do;
  */
    /*Run all INDY9 tables*/
/*    %run_indy9_tables;
  %end;
*/
  %else %if &main_menu=8 %then %do;
    /*Run batch*/
    %run_batch_tables;
  %end;
%end;

%mend;
/*
%tabeling_menu;
*/
