
/*Setting up forest types*/
%macro typgrpcd_def(direction);

%if "&use_idb." = "" %then %do;
%validate_datasource;
  %if %upcase("&direction")="COL" %then
    %let col_var_name_out=Forest type group;

  proc sql noprint;
    create table all_forest_types as
    select distinct f.value, f.typgrpcd
	  	  from &ftype. f,
           &cond. c,
		   ppsa_temp ppsa
		where f.value = c.fortypcd and
		      c.statecd  = ppsa.statecd and
			  c.plot     = ppsa.plot and
			  f.value ne 999 and
			  ppsa.evalid in (&evalid.)	;
   quit;

  proc sql noprint;
    create table all_forest_type_groups as
	select q.typgrpcd, f.meaning
	  from (
      select distinct f.typgrpcd
	       from &ftype. f,
             &cond. c,
		     ppsa_temp ppsa
		  where f.value = c.fortypcd and
		        c.statecd  = ppsa.statecd and
			    c.plot     = ppsa.plot and
			    f.value ne 999 and
			    ppsa.evalid in (&evalid.)	
		) as q,
		&ftype. f
		where q.typgrpcd = f.value;
   quit;
  %let num_forest_type_groups=&sqlobs.;

  proc sql noprint;
	select f.typgrpcd, f.meaning
	  into :type_group_code1 - :type_group_code&num_forest_type_groups., 
           :type_meaning1 - :type_meaning&num_forest_type_groups.
	  from all_forest_type_groups f
	  ;
  quit;

  %let run_counter = 1;
  %let break = 0;
  %do i = 1 %to &num_forest_type_groups.;
    
    proc sql noprint;
	  select ft.value
	    into :temp separated by ','
	      from all_forest_types ft
		    where ft.typgrpcd in (&&type_group_code&i.)
	    ;
    quit;

    %if &&type_group_code&i.>=500 and &break. = 0 %then %do;
	  %let break=1;
      %let &&direction._rest&run_counter.x=c.fortypcd <500;
      %let &&direction._description&run_counter.=All softwood types;
	  %let run_counter=%eval(&run_counter. + 1);
	%end;

    %let &&direction._rest&run_counter.x=c.fortypcd in (&temp.);
    %let &&direction._description&run_counter.=&&type_meaning&i.;
	%let run_counter=%eval(&run_counter. + 1);
  %end;

  %let &&direction._rest&run_counter.x=c.fortypcd >=500 and c.fortypcd <999;
  %let &&direction._description&run_counter.=All hardwood types;
  %let run_counter=%eval(&run_counter. + 1);

  %let &&direction._rest&run_counter.x=c.fortypcd =999;
  %let &&direction._description&run_counter.=Nonstocked;
  %let run_counter=%eval(&run_counter. + 1);

  %let &&direction._rest&run_counter.x=c.fortypcd ne .;
  %let &&direction._description&run_counter.=All forest types;

  %let max_&direction._num=&run_counter.;

%end;
%else %do;
/*use IDB*/

%validate_datasource;
  %if %upcase("&direction")="COL" %then
    %let col_var_name_out=Forest type group;


  proc sql noprint;
    create table all_forest_types as
    select distinct c.for_type, f.fiadb_forest_type_group
	  	  from &ftype. f,
               &cond. c
		where f.fia_forest_type = c.for_type and
			  c.for_type not in (43,45,99, 888, 999)
	  group by f.fiadb_forest_type_group
    ;
   quit;

  proc sql noprint;
    create table all_forest_type_groups as
      select distinct f.fiadb_forest_type_group, f.fiadb_forest_type_group_name
	       from &ftype. f,
                &cond. c
		where f.fia_forest_type = c.for_type and
			  c.for_type not in (43,45,99, 888, 999)
           group by fiadb_forest_type_group
			  ;		
   quit;
  %let num_forest_type_groups=&sqlobs.;

  proc sql noprint;
	select f.fiadb_forest_type_group, f.fiadb_forest_type_group_name
	  into :type_group_code1 - :type_group_code&num_forest_type_groups., 
           :type_meaning1 - :type_meaning&num_forest_type_groups.
	  from all_forest_type_groups f
	  group by fiadb_forest_type_group
	  ;
  quit;

  %let run_counter = 1;
  %let break = 0;
  %do i = 1 %to &num_forest_type_groups.;

    proc sql noprint;
	  select ft.for_type
	    into :temp separated by ','
	      from all_forest_types ft
		    where ft.fiadb_forest_type_group in (&&type_group_code&i.)
	    ;
    quit;

    %if &&type_group_code&i.>=400 and &break. = 0 %then %do;
	  %let break=1;
      %let &&direction._rest&run_counter.x=c.for_type <300 and c.for_type not in (43,45,99, 888, 999);
      %let &&direction._description&run_counter.=All softwood types;
	  %let run_counter=%eval(&run_counter. + 1);
	%end;

    %let &&direction._rest&run_counter.x=c.for_type in (&temp.);
    %let &&direction._description&run_counter.=&&type_meaning&i.;
	%let run_counter=%eval(&run_counter. + 1);
  %end;

  %let &&direction._rest&run_counter.x=c.for_type >=300 and c.for_type not in (43,45,99, 888, 999);
  %let &&direction._description&run_counter.=All hardwood types;
  %let run_counter=%eval(&run_counter. + 1);

  %let &&direction._rest&run_counter.x=c.for_type =999;
  %let &&direction._description&run_counter.=Nonstocked;
  %let run_counter=%eval(&run_counter. + 1);

  %let &&direction._rest&run_counter.x=c.for_type in (43,888);
  %let &&direction._description&run_counter.=Unknown;
  %let run_counter=%eval(&run_counter. + 1);

  %let &&direction._rest&run_counter.x=c.for_type ne .;
  %let &&direction._description&run_counter.=All forest types;
  %let run_counter=%eval(&run_counter. + 1);

  %let &&direction._rest&run_counter.x=c.for_type =45;
  %let &&direction._description&run_counter.=Chaparral;

  %let max_&direction._num=&run_counter.;



%end;
%mend;


* %let direction=col;
%macro fortypcd_def(direction);
  %if %upcase("&direction")="COL" %then
    %let col_var_name_out=Forest types;

  proc sql noprint;
    create table all_forest_types as
    select distinct f.value, f.meaning
	  	  from &ftype. f,
           &cond. c,
		   ppsa_temp ppsa
		where f.value = c.fortypcd and
		      c.statecd  = ppsa.statecd and
			  c.plot     = ppsa.plot and
			  f.value ne 999 and
			  ppsa.evalid in (&evalid.)	;
   quit;
  %let num_forest_types=&sqlobs.;

  data all_forest_types;
    set all_forest_types;
	  restriction="c.fortypcd =" || input(value,$100. );
  run;

  proc sql noprint;
    create table _null_ as
      select value
	  	  from all_forest_types
		where value <500;
   quit;
  %let num_softwoods=&sqlobs.;

  proc sql noprint;
    select restriction, meaning 
	  into :&&direction._rest1-:&&direction._rest&num_softwoods.,
	       :&&direction._description1-:&&direction._description&num_softwoods.
	  	  from all_forest_types
		where value <500;
   quit;
   
   %do i=1 %to &num_softwoods;
     %let &&direction._rest&i.x =&&&direction._rest&i.;
   %end;
%put &row_rest1. &row_rest1x. &num_softwoods.;
   

   %let &&direction._rest%eval(&num_softwoods.+1)x=c.fortypcd <500;
   %let &&direction._description%eval(&num_softwoods.+1)=All softwood types;


  proc sql noprint;
    create table _null_ as
      select value
	  	  from all_forest_types
		where value >=500 and value<999;
   quit;
  %let num_hardwoods=&sqlobs.;
/*
  %put &num_hardwoods.;
*/
  proc sql noprint;
    select restriction, meaning 
	  into :&&direction._rest%eval(&num_softwoods.+2)-
            :&&direction._rest%eval(&num_softwoods.+&num_hardwoods.+2),
	       :&&direction._description%eval(&num_softwoods.+2)-
            :&&direction._description%eval(&num_softwoods.+&num_hardwoods.+2)
	  	  from all_forest_types
		where value >=500 and value<999;
   quit;
   
   %do i=%eval(&num_softwoods.+2) %to %eval(&num_softwoods.+&num_hardwoods.+2);
     %let &&direction._rest&i.x =&&&direction._rest&i.;
   %end;
   
  %let &&direction._rest%eval(&num_softwoods.+&num_hardwoods.+2)x=c.fortypcd >=500 and c.fortypcd <999;
  %let &&direction._description%eval(&num_softwoods.+&num_hardwoods.+2)=All hardwood types;

  %let &&direction._rest%eval(&num_softwoods.+&num_hardwoods.+3)x=c.fortypcd =999;
  %let &&direction._description%eval(&num_softwoods.+&num_hardwoods.+3)=Nonstocked;

  %let &&direction._rest%eval(&num_softwoods.+&num_hardwoods.+4)x=c.fortypcd ne .;
  %let &&direction._description%eval(&num_softwoods.+&num_hardwoods.+4)=All forest types;

  %let max_&direction._num=%eval(&num_softwoods.+&num_hardwoods.+4);


%mend;
/*
%fortypcd_def(row);
*/
