*options mlogic mprint symbolgen;
*options nomlogic nomprint nosymbolgen;


/*Setting up Species groups*/
%macro spgrpcd_def(direction);
%global forest_type forest_type_name;

/*Get all possible Species Groups first
*/
%validate_datasource;
  %if %upcase("&direction")="COL" %then
    %let col_var_name_out=Species group;

  proc sql noprint;
    create table all_species as
      select distinct sp.spgrpcd, sp.name
	  from &species_group. sp,
           &attribute_tablex. t,
		   &ppsa. ppsa
		where sp.spgrpcd = t.spgrpcd and
		      t.statecd  = ppsa.statecd and
			  t.plot     = ppsa.plot and
			  ppsa.evalid in (&evalid.)	
      ;
  quit;

/*All Softwoods*/
  proc sql noprint;
    select sp.spgrpcd
	  into :softwoods_temp separated by "&delimiter."
	  from all_species sp
         where sp.spgrpcd < 25 
      ;
  quit;
  %let num_softwoods=&sqlobs.;

  proc sql noprint;
    select sp.spgrpcd
	  into :all_softwoods_temp separated by ","
	  from all_species sp
         where sp.spgrpcd < 25 
      ;
  quit;


/*All Hardwoods*/
  proc sql noprint;
    select sp.spgrpcd
	  into :hardwoods_temp separated by "&delimiter."
	  from all_species sp
         where sp.spgrpcd >= 25 
      ;
  quit;
  %let num_hardwoods=&sqlobs.;

  proc sql noprint;
    select sp.spgrpcd
	  into :all_hardwoods_temp separated by ","
	  from all_species sp
         where sp.spgrpcd >= 25 
      ;
  quit;

      ;
  quit;
/*All forest types*/
  proc sql noprint;
    select sp.spgrpcd
	  into :all_temp separated by ","
	  from all_species sp
      ;
  quit;


  proc sql noprint;
    select sp.name
	  into :softwoods_name_temp separated by "&delimiter."
	  from all_species sp
         where sp.spgrpcd < 25 
      ;
  quit;

  proc sql noprint;
    select sp.name
	  into :hardwoods_name_temp separated by "&delimiter."
	  from all_species sp
         where sp.spgrpcd >= 25 
      ;
  quit;

  %let run_counter = 1;
  %do i = 1 %to &num_softwoods.;
    %let &direction._rest&run_counter.x=t.spgrpcd in (%scan(%bquote(&softwoods_temp.), &i.,"&delimiter."));
    %let &direction._description&run_counter.=%scan(%bquote(&softwoods_name_temp.), &i.,"&delimiter.");
	%let run_counter=%eval(&run_counter. + 1);
  %end;

  %let &direction._rest&run_counter.x=t.spgrpcd in (&all_softwoods_temp.);
  %let &direction._description&run_counter.=Total Softwoods;
  %let run_counter=%eval(&run_counter. + 1);

  %do i = 1 %to &num_hardwoods.;
    %let &direction._rest&run_counter.x=t.spgrpcd in (%scan(%bquote(&hardwoods_temp.), &i.,"&delimiter."));
    %let &direction._description&run_counter.=%scan(%bquote(&hardwoods_name_temp.), &i.,"&delimiter.");
	%let run_counter=%eval(&run_counter. + 1);
  %end;

  %let &direction._rest&run_counter.x=t.spgrpcd in (&all_hardwoods_temp.);
  %let &direction._description&run_counter.=Total Hardwoods;
  %let run_counter=%eval(&run_counter. + 1);

  %let &direction._rest&run_counter.x=t.spgrpcd in (&all_temp.);
  %let &direction._description&run_counter.=All Species Groups;

  %let max_&direction._num=%eval(&run_counter.);

%mend;



